@@ -6,7 +6,7 @@
 //
 //
 #if RELEASE
-static char RCSid[] = "$Id: defaults.cc,v 1.43.2.15 2000/02/15 22:08:34 grdetil Exp $";
+static char RCSid[] = "$Id: defaults.cc,v 1.43.2.36 2002/01/25 04:44:33 ghutchis Exp $";
 #endif
 
 #include "Configuration.h"
@@ -27,15 +27,20 @@ ConfigDefaults	defaults[] =
     //
     // General defaults
     //
+    {"accents_db",			"${database_base}.accents.db"},
     {"add_anchors_to_excerpt",		"true"},
     {"allow_in_form",			""},
     {"allow_numbers",			"false"},
     {"allow_virtual_hosts",		"true"},
+    {"anchor_target",			""},
+    {"any_keywords",			"false"},
     {"authorization",			""}, 
     {"backlink_factor",                 "1000"},
-    {"bad_extensions",			".wav .gz .z .sit .au .zip .tar .hqx .exe .com .gif .jpg .jpeg .aiff .class .map .ram .tgz .bin .rpm .mpg .mov .avi"},
+    {"bad_extensions",			".wav .gz .z .sit .au .zip .tar .hqx .exe .com .gif .jpg .jpeg .aiff .class .map .ram .tgz .bin .rpm .mpg .mov .avi .css"},
     {"bad_querystr",                    ""},
     {"bad_word_list",			"${common_dir}/bad_words"},
+    {"boolean_keywords",		"and or not"},
+    {"boolean_syntax_errors",		"Expected 'a search word' 'at the end' 'instead of' 'end of expression'"},
     {"build_select_lists",		""},
     {"case_sensitive",                  "true"},
     {"common_url_parts",                "http:// http://www. ftp:// ftp://ftp. /pub/ .html .htm .gif .jpg .jpeg /index.html /index.htm .com/ .com mailto:"},
@@ -46,17 +51,22 @@ ConfigDefaults	defaults[] =
     {"date_format",                     ""},
     {"database_base",			"${database_dir}/db"},
     {"description_factor",              "150"},
+    {"description_meta_tag_names",	"description"},
     {"doc_db",				"${database_base}.docdb"},
     {"doc_index",			"${database_base}.docs.index"},
     {"doc_list",			"${database_base}.docs"},
+    {"endday",				""},
     {"end_ellipses",			"<b><tt> ...</tt></b>"},
     {"end_highlight",			"</strong>"},
     {"endings_affix_file",		"${common_dir}/english.aff"},
     {"endings_dictionary",		"${common_dir}/english.0"},
     {"endings_root2word_db",		"${common_dir}/root2word.db"},
     {"endings_word2root_db",		"${common_dir}/word2root.db"},
+    {"endmonth",			""},
+    {"endyear",				""},
     {"excerpt_length",			"300"},
     {"excerpt_show_top",		"false"},
+    {"exclude",			        ""},
     {"exclude_urls",			"/cgi-bin/ .cgi"},
     {"external_parsers",		""},
     {"extra_word_characters",		""},
@@ -67,8 +77,14 @@ ConfigDefaults	defaults[] =
     {"heading_factor_5",		"1"},
     {"heading_factor_6",		"0"},
     {"htnotify_sender",			"webmaster@www"},
+    {"htnotify_replyto",		""},
+    {"htnotify_webmaster",		"ht://Dig Notification Service"},
+    {"htnotify_prefix_file",		""},
+    {"htnotify_suffix_file",		""},
     {"http_proxy",			""},
     {"http_proxy_exclude",		""},
+    {"ignore_alt_text",			"false"},
+    {"ignore_dead_servers",		"true"},
     {"image_list",			"${database_base}.images"},
     {"iso_8601",                        "false"},
     {"keywords_factor",			"100"},
@@ -87,12 +103,14 @@ ConfigDefaults	defaults[] =
     {"max_description_length",		"60"},
     {"max_descriptions",                "5"},
     {"max_doc_size",			"100000"},
+    {"max_excerpts",                    "1" },
     {"max_head_length",			"512"},
     {"max_hop_count",			"999999"},
     {"max_keywords",			"-1"},
     {"max_meta_description_length",     "512"},
     {"max_prefix_matches",		"1000"},
     {"max_stars",			"4"},
+    {"maximum_page_buttons",		"${maximum_pages}"},
     {"maximum_pages",			"10"},
     {"maximum_word_length",		"12"},
     {"metaphone_db",			"${database_base}.metaphone.db"},
@@ -100,7 +118,8 @@ ConfigDefaults	defaults[] =
     {"method_names",			"and All or Any boolean Boolean"},
     {"minimum_word_length",		"3"},
     {"minimum_prefix_length",		"1"},
-    {"modification_time_is_now",        "false"},
+    {"modification_time_is_now",        "true"},
+    {"multimatch_factor",		"1"},
     {"next_page_text",			"[next]"},
     {"no_excerpt_text",			"<em>(None of the search words were found in the top of this document.)</em>"},
     {"no_excerpt_show_top",             "false"},
@@ -116,16 +135,20 @@ ConfigDefaults	defaults[] =
     {"page_list_header",		"<hr noshade size=2>Pages:<br>"},
     {"page_number_separator",		"\" \""},
     {"page_number_text",		""},
+    {"plural_suffix",			"s"},
     {"prefix_match_character",		"*"},
     {"prev_page_text",			"[prev]"},
     {"remove_bad_urls",			"true"},
     {"remove_default_doc",              "index.html"},
+    {"restrict",                        ""},
     {"robotstxt_name",			"htdig"},
     {"script_name",                     ""},
     {"search_algorithm",		"exact:1"},
+    {"search_results_contenttype",	"text/html"},
     {"search_results_footer",		"${common_dir}/footer.html"},
     {"search_results_header",		"${common_dir}/header.html"},
     {"search_results_wrapper",		""},
+    {"search_rewrite_rules",		""},
     {"server_aliases",                  ""},
     {"server_wait_time",                "0"},
     {"server_max_docs",                 "-1"},
@@ -135,9 +158,12 @@ ConfigDefaults	defaults[] =
     {"star_blank",			"${image_url_prefix}/star_blank.gif"},
     {"star_image",			"${image_url_prefix}/star.gif"},
     {"star_patterns",			""},
+    {"startday",			""},
     {"start_ellipses",			"<b><tt>... </tt></b>"},
     {"start_highlight",			"<strong>"},
+    {"startmonth",			""},
     {"start_url",			"http://www.htdig.org/"},
+    {"startyear",			""},
     {"substring_max_words",		"25"},
     {"synonym_db",			"${common_dir}/synonyms.db"},
     {"synonym_dictionary",		"${common_dir}/synonyms"},
@@ -148,13 +174,16 @@ ConfigDefaults	defaults[] =
     {"text_factor",			"1"},
     {"timeout",				"30"},
     {"title_factor",			"100"},
-    {"translate_amp",                   "false"},
-    {"translate_lt_gt",                 "false"},
-    {"translate_quot",                  "false"},
-    {"url_list",			"${database_base}.urls"},
-    {"url_part_aliases",                ""},
-    {"url_log",				"${database_base}.log"},
+    {"translate_amp",                   "true"},
+    {"translate_latin1",                "true"},
+    {"translate_lt_gt",                 "true"},
+    {"translate_quot",                  "true"},
     {"uncoded_db_compatible",		"true"},
+    {"url_list",			"${database_base}.urls"},
+    {"url_log",				"${database_base}.log"},
+    {"url_part_aliases",		""},
+    {"url_rewrite_rules",		""},
+    {"use_doc_date",			"false"},
     {"use_star_image",			"true"},
     {"use_meta_description",            "false"},
     {"user_agent",			"htdig"},
