@@ -1,31 +1,41 @@
 //
 // HTML.cc
 //
-// Implementation of HTML
-// Class to parse HTML documents and return useful information to the Retriever
+// HTML: Class to parse HTML documents and return useful information
+//       to the Retriever
 //
+// Part of the ht://Dig package   <http://www.htdig.org/>
+// Copyright (c) 1995-2001 The ht://Dig Group
+// For copyright details, see the file COPYING in your distribution
+// or the GNU Public License version 2 or later
+// <http://www.gnu.org/copyleft/gpl.html>
 //
 #if RELEASE
-static char RCSid[] = "$Id: HTML.cc,v 1.30.2.14 2000/02/15 20:11:29 grdetil Exp $";
+static char RCSid[] = "$Id: HTML.cc,v 1.30.2.24 2002/01/09 22:12:29 grdetil Exp $";
 #endif
 
 #include "htdig.h"
 #include "HTML.h"
 #include "SGMLEntities.h"
 #include "Configuration.h"
-#include <ctype.h>
 #include "StringMatch.h"
 #include "StringList.h"
 #include "URL.h"
 #include "HtWordType.h"
 
+// Flags for noindex & nofollow, indicating who turned indexing off/on...
+#define TAGnoindex		0x0001
+#define TAGstyle		0x0002
+#define TAGscript		0x0004
+#define TAGmeta_htdig_noindex	0x0008
+#define TAGmeta_robots		0x0010
+
 static StringMatch	tags;
 static StringMatch	nobreaktags;
 static StringMatch	spacebeforetags;
 static StringMatch	spaceaftertags;
-static StringMatch	attrs;
-static StringMatch	srcMatch;
-static StringMatch	hrefMatch;
+static StringMatch	metadatetags;
+static StringMatch	descriptionMatch;
 static StringMatch	keywordsMatch;
 static int		keywordsCount;
 static int		max_keywords;
@@ -40,7 +50,7 @@ static int		totlength;
 #define ADDSPACE(in_space)	\
     if (!in_space)							\
     {									\
-	if (in_title && doindex)					\
+	if (in_title && !noindex)					\
 	{								\
 	    title << ' ';						\
 	}								\
@@ -48,7 +58,7 @@ static int		totlength;
 	{								\
 	    description << ' ';						\
 	}								\
-	if (head.length() < max_head_length && doindex && !in_title)	\
+	if (head.length() < max_head_length && !noindex && !in_title)	\
 	{								\
 	    head << ' ';						\
 	}								\
@@ -64,19 +74,9 @@ HTML::HTML()
     //
     // Initialize the patterns that we will try to match.
     // The tags Match object is used to match tag commands while
-    // the attrs Match object is used to match names of tag parameters.
     //
     tags.IgnoreCase();
-    tags.Pattern("title|/title|a|/a|h1|h2|h3|h4|h5|h6|/h1|/h2|/h3|/h4|/h5|/h6|noindex|/noindex|img|li|meta|frame|area|base|embed|object|link|style|/style");
-
-    attrs.IgnoreCase();
-    attrs.Pattern("src|href|name");
-
-    srcMatch.IgnoreCase();
-    srcMatch.Pattern("src");
-
-    hrefMatch.IgnoreCase();
-    hrefMatch.Pattern("href");
+    tags.Pattern("title|/title|a|/a|h1|h2|h3|h4|h5|h6|/h1|/h2|/h3|/h4|/h5|/h6|noindex|/noindex|img|li|meta|frame|area|base|embed|object|link|style|/style|script|/script");
 
     // These tags don't cause a word break.  They may also be in "tags" above,
     // except for the "a" tag, which must be handled as a special case.
@@ -91,11 +91,17 @@ HTML::HTML()
     spaceaftertags.IgnoreCase();
     spaceaftertags.Pattern("/title|/h1|/h2|/h3|/h4|/h5|/h6|/address|/blockquote");
 
-    //String	keywordNames = config["keywords_meta_tag_names"];
-    //keywordNames.replace(' ', '|');
-    //keywordNames.remove(",\t\r\n");
-    //keywordsMatch.IgnoreCase();
-    //keywordsMatch.Pattern(keywordNames);
+    // These are the name values of meta tags that carry date information.
+    metadatetags.IgnoreCase();
+    metadatetags.Pattern("date|dc.date|dc.date.created|dc.data.modified");
+
+    // These are the name values of meta tags that carry descriptions.
+    StringList descrNames(config["description_meta_tag_names"], " \t");
+    descriptionMatch.IgnoreCase();
+    descriptionMatch.Pattern(descrNames.Join('|'));
+    descrNames.Release();
+
+    // These are the name values of meta tags that carry keywords.
     StringList keywordNames(config["keywords_meta_tag_names"], " \t");
     keywordsMatch.IgnoreCase();
     keywordsMatch.Pattern(keywordNames.Join('|'));
@@ -115,8 +121,8 @@ HTML::HTML()
     in_ref = 0;
     in_heading = 0;
     base = 0;
-    doindex = 1;
-    dofollow = 1;
+    noindex = 0;
+    nofollow = 0;
     minimumWordLength = config.Value("minimum_word_length", 3);
 }
 
@@ -154,14 +160,16 @@ HTML::parse(Retriever &retriever, URL &baseURL)
     unsigned char       *ptext = text;
     static char         *skip_start = config["noindex_start"];
     static char         *skip_end = config["noindex_end"];
+    int			skip_start_len = strlen(skip_start);
+    int			skip_end_len = strlen(skip_end);
 
     keywordsCount = 0;
     offset = 0;
     title = 0;
     head = 0;
     meta_dsc = 0;
-    doindex = 1;
-    dofollow = 1;
+    noindex = 0;
+    nofollow = 0;
     in_heading = 0;
     in_title = 0;
     in_ref = 0;
@@ -176,13 +184,13 @@ HTML::parse(Retriever &retriever, URL &baseURL)
       // This can contain any HTML. 
       //
       if (*skip_start &&
-	  mystrncasecmp((char *)position, skip_start, strlen(skip_start)) == 0)
+	  mystrncasecmp((char *)position, skip_start, skip_start_len) == 0)
 	{
 	  q = (unsigned char*)mystrcasestr((char *)position, skip_end);
 	  if (!q)
 	    *position = '\0';       // Rest of document will be skipped...
 	  else
-	    position = q + strlen(skip_end);
+	    position = q + skip_end_len;
 	  continue;
 	}
 
@@ -229,7 +237,7 @@ HTML::parse(Retriever &retriever, URL &baseURL)
 	    {
 	      // Not a comment declaration after all
 	      // but possibly DTD: get to the end
-	      q = (unsigned char*)strstr((char *)position, ">");
+	      q = (unsigned char*)strchr((char *)position, '>');
 	      if (q)
 		{
 		  position = q + 1;
@@ -266,7 +274,11 @@ HTML::parse(Retriever &retriever, URL &baseURL)
            *ptext = SGMLEntities::translateAndUpdate(position);
            if (*ptext == '<') 
            {
-              *ptext = ' ';
+              // got a decoded &lt;, make a fake tag for it
+              // to avoid confusing it with real tag start
+              *ptext++ = '<';
+              *ptext++ = '~';
+              *ptext = '>';
            }
            ptext++;
         }
@@ -286,7 +298,7 @@ HTML::parse(Retriever &retriever, URL &baseURL)
 	offset = position - start;
 	// String = 0 is expensive
 	// word = 0;
-	if (*position == '<')
+	if (*position == '<' && (position[1] != '~' || position[2] != '>'))
 	  {
 	    //
 	    // Start of a tag.  Since tags cannot be nested, we can simply
@@ -295,9 +307,9 @@ HTML::parse(Retriever &retriever, URL &baseURL)
 	    q = (unsigned char*)strchr((char *)position, '>');
 	    if (!q)
 	      break; // Syntax error in the doc.  Tag never ends.
-	    tag = 0;
-	    tag.append((char*)position, q - position + 1);
 	    position++;
+	    tag = 0;
+	    tag.append((char*)position, q - position);
 	    while (isspace(*position))
 		position++;
 	    if (!in_space && spacebeforetags.CompareWord((char *)position)
@@ -333,6 +345,9 @@ HTML::parse(Retriever &retriever, URL &baseURL)
 	    while (*position && HtIsWordChar(*position))
 	    {
 		word << (char)*position;
+		// handle case where '<' is in extra_word_characters...
+		if (strncmp((char *)position, "<~>", 3) == 0)
+		    position += 2;      // skip over fake tag for decoded '<'
 		position++;
 		if (*position == '<')
 		{
@@ -347,8 +362,9 @@ HTML::parse(Retriever &retriever, URL &baseURL)
 			q = (unsigned char*)strchr((char *)position, '>');
 			if (q)
 			{
+			    position++;
 			    tag = 0;
-			    tag.append((char*)position, q - position + 1);
+			    tag.append((char*)position, q - position);
 			    do_tag(retriever, tag);
 			    position = q+1;
 			}
@@ -356,7 +372,7 @@ HTML::parse(Retriever &retriever, URL &baseURL)
 		}
 	    }
 
-	    if (in_title && doindex)
+	    if (in_title && !noindex)
 	    {
 		title << word;
 	    }
@@ -370,14 +386,14 @@ HTML::parse(Retriever &retriever, URL &baseURL)
 		else
 		{
 		    description << " ...";
-		    if (dofollow)
+		    if (!nofollow)
 		      retriever.got_href(*href, description);
 		    in_ref = 0;
 		    description = 0;
 		}
 	    }
 
-	    if (head.length() < max_head_length && doindex && !in_title)
+	    if (head.length() < max_head_length && !noindex && !in_title)
 	    {
 		//
 		// Capitalize H1 and H2 blocks
@@ -394,7 +410,7 @@ HTML::parse(Retriever &retriever, URL &baseURL)
 		  head << word;
 	    }
 
-	    if (word.length() >= minimumWordLength && doindex)
+	    if (word.length() >= minimumWordLength && !noindex)
 	    {
 	      retriever.got_word(word,
 				 int(offset * 1000 / totlength),
@@ -416,7 +432,7 @@ HTML::parse(Retriever &retriever, URL &baseURL)
 		//
 		// Not whitespace
 		//
-		if (head.length() < max_head_length && doindex && !in_title)
+		if (head.length() < max_head_length && !noindex && !in_title)
 		{
 		    // We don't want to add random chars to the 
 		    // excerpt if we're in the title.
@@ -426,19 +442,22 @@ HTML::parse(Retriever &retriever, URL &baseURL)
 		{
 		    description << *position;
 		}
-		if (in_title && doindex)
+		if (in_title && !noindex)
 		{
 		    title << *position;
 		}
 		in_space = 0;
 		in_punct = 1;
+		// handle normal case where decoded '<' is punctuation...
+		if (strncmp((char *)position, "<~>", 3) == 0)
+		    position += 2;      // skip over fake tag for decoded '<'
 	    }
 	    position++;
 	}
     }
     retriever.got_head(head);
 
-    delete text;
+    delete [] text;
 }
 
 
@@ -448,8 +467,8 @@ HTML::parse(Retriever &retriever, URL &baseURL)
 void
 HTML::do_tag(Retriever &retriever, String &tag)
 {
-    char	*position = tag.get() + 1;		// Skip the '<'
-    char	*q, *t;
+    static int	ignore_alt_text = config.Boolean("ignore_alt_text", 0);
+    char	*position = tag.get();
     int		which, length;
 
     while (isspace(*position))
@@ -459,8 +478,13 @@ HTML::do_tag(Retriever &retriever, String &tag)
     if (tags.CompareWord(position, which, length) < 0)
 	return; // Nothing matched.
 
+    // Use the configuration code to match attributes as key-value pairs
+    Configuration	attrs;
+    attrs.NameValueSeparators("=");
+    attrs.Add(position);
+
     if (debug > 3)
-	cout << "Tag: " << position << ", matched " << which << endl;
+	cout << "Tag: <" << tag << ">, matched " << which << endl;
     
     switch (which)
     {
@@ -486,139 +510,51 @@ HTML::do_tag(Retriever &retriever, String &tag)
 			
 	case 2:		// "a"
 	{
-	    which = -1;
-	    int pos;
-	    while ((pos = attrs.FindFirstWord(position, which, length)) >= 0)
+	  if (attrs["href"])
 	    {
-		position += pos + length;
-		if (debug > 1)
-		    cout << "A tag: pos = " << pos << ", position = " << position << endl;
-		switch (which)
+	      //
+	      // a href seen
+	      //
+	      if (in_ref)
 		{
-		    case 1:		// "href"
-		    {
-			//
-			// a href seen
-			//
-			while (*position && *position != '=')
-			    position++;
-			if (!*position)
-			    return;
-			position++;
-			while (isspace(*position))
-			    position++;
-                       //
-		       // Allow either single quotes or double quotes
-                       // around the URL itself
-                       //
-                       if (*position == '"'||*position == '\'')
-			{
-			    position++;
-			    q = strchr(position, position[-1]);
-			    if (!q)
-				break;
-                           //
-                           // We seem to have matched the opening quote char
-                           // Mark the end of the quotes as our endpoint, so
-                           // that we can continue parsing after the current
-                           // text
-                           //
-                           *q = '\0';
-                           //
-                           // If a '#' is present in a quoted URL,
-                           //  treat that as the end of the URL, but we skip
-                           //  past the quote to parse the rest of the anchor.
-                           //
-                           if ((t = strchr(position, '#')) != NULL)
-                               *t = '\0';
-			}
-			else
-			{
-			    q = position;
-			    while (*q &&
-				   *q != '>' &&
-				   !isspace(*q) && // *q != '?'  ???? -grh
-				   *q != '#')
-				q++;
-			    *q = '\0';
-			}
-			if (in_ref)
-			{
-			    if (debug > 1)
-				cout << "Terminating previous <a href=...> tag,"
-				     << " which didn't have a closing </a> tag."
-				     << endl;
-			    if (dofollow)
-				retriever.got_href(*href, description);
-			    in_ref = 0;
-			}
-			delete href;
-			href = new URL(transSGML(position), *base);
-			in_ref = 1;
-			description = 0;
-			position = q + 1;
-			break;
-		    }
-
-		    case 2:		// "name"
-		    {
-			//
-			// a name seen
-			//
-			while (*position && *position != '=')
-			    position++;
-			if (!*position)
-			    return;
-			position++;
-			while (isspace(*position))
-			    position++;
-                       //
-                       // Allow either single quotes or double quotes
-                       // around the URL itself
-                       //
-                       if (*position == '"'||*position == '\'')
-			{
-			    position++;
-			    q = strchr(position, position[-1]);
-			    if (!q)
-				break;
-                           //
-                           // We seem to have matched the opening quote char
-                           // Mark the end of the quotes as our endpoint, so
-                           // that we can continue parsing after the current
-                           // text
-                           //
-                           *q = '\0';
-                           //
-                           // If a '#' is present in a quoted URL,
-                           //  treat that as the end of the URL, but we skip
-                           //  past the quote to parse the rest of the anchor.
-                           //
-                           if ((t = strchr(position, '#')) != NULL)
-                               *t = '\0';
-			}
-			else
-			{
-			    q = position;
-			    while (*q && *q != '>' && !isspace(*q))
-				q++;
-			*q = '\0';
-			}
-			retriever.got_anchor(transSGML(position));
-			position = q + 1;
-			break;
-		    }
-		    default:
-			break;
+		  if (debug > 1)
+		    cout << "Terminating previous <a href=...> tag,"
+			 << " which didn't have a closing </a> tag."
+			 << endl;
+		  if (!nofollow)
+		      retriever.got_href(*href, description);
+		  in_ref = 0;
 		}
+	      if (href)
+		delete href;
+	      href = new URL(transSGML(attrs["href"]), *base);
+	      in_ref = 1;
+	      description = 0;
+	      break;
+	    }
+	  
+	  if (attrs["title"] && attrs["href"])
+	    {
+	      //
+	      // a title seen for href
+	      //
+	      retriever.got_href(*href, transSGML(attrs["title"]));
 	    }
-	    break;
-	}
 
+	  if (attrs["name"])
+	    {
+	      //
+	      // a name seen
+	      //
+	      retriever.got_anchor(transSGML(attrs["name"]));
+	    }
+	  break;
+	}
+				   
 	case 3:		// "/a"
 	    if (in_ref)
 	    {
-	      if (dofollow)
+	      if (!nofollow)
 		retriever.got_href(*href, description);
 	      in_ref = 0;
 	    }
@@ -658,133 +594,99 @@ HTML::do_tag(Retriever &retriever, String &tag)
 	    break;
 
 	case 16:	// "noindex"
+	    noindex |= TAGnoindex;
+	    nofollow |= TAGnoindex;
+	    if (attrs["follow"])
+		nofollow &= ~TAGnoindex;
+	    break;
+
 	case 27:	// "style"
-	    doindex = 0;
-	    dofollow = 0;
+	    noindex |= TAGstyle;
+	    nofollow |= TAGstyle;
+	    break;
+
+        case 29:        // "script"
+	    noindex |= TAGscript;
+	    nofollow |= TAGscript;
 	    break;
 
 	case 17:	// "/noindex"
+	    noindex &= ~TAGnoindex;
+	    nofollow &= ~TAGnoindex;
+	    break;
+
 	case 28:	// "/style"
-	    doindex = 1;
-	    dofollow = 1;
+	    noindex &= ~TAGstyle;
+	    nofollow &= ~TAGstyle;
+	    break;
+
+        case 30:	// "/script"
+	    noindex &= ~TAGscript;
+	    nofollow &= ~TAGscript;
 	    break;
 
 	case 18:	// "img"
-	{
-	    // Handle alt parameter
-	    Configuration	conf;
-	    conf.NameValueSeparators("=");
-	    conf.Add(position+length);
-	    if (conf["alt"])
-	    {
-		char	*alttxt = transSGML(conf["alt"]);
-		if (doindex && in_title)
+	  {
+	    if (!ignore_alt_text && attrs["alt"])
+	      {
+		char	*alttxt = transSGML(attrs["alt"]);
+		if (!noindex && in_title)
 		    title << alttxt << " ";
 		if (in_ref && description.length() < max_description_length)
 		    description << alttxt << " ";
-		if (doindex && !in_title && head.length() < max_head_length)
+		if (!noindex && !in_title && head.length() < max_head_length)
 		    head << alttxt << " ";
-		char	*w = HtWordToken(alttxt);
-		while (w && doindex)
-		{
+		char *w = HtWordToken(alttxt);
+		while (w && !noindex)
+		  {
 		    if (strlen(w) >= minimumWordLength)
 		      retriever.got_word(w,
 				 int((offset+(w-alttxt)) * 1000
 					/ totlength),
 				 in_heading);
 		    w = HtWordToken(0);
-		}
+		  }
 		w = '\0';
-	    }
-
-	    // Handle src parameter
-	    which = -1;
-	    int pos = attrs.FindFirstWord(position, which, length);
-	    if (pos < 0 || which != 0)
-		break;
-	    position += pos + length;
-	    while (*position && *position != '=')
-		position++;
-	    if (!*position)
-		break;
-	    position++;
-	    while (isspace(*position))
-		position++;
-           //
-           // Allow either single quotes or double quotes
-           // around the URL itself
-           //
-           if (*position == '"'||*position == '\'')
-	    {
-		position++;
-		q = strchr(position, position[-1]);
-		if (!q)
-		    break;
-               //
-               // We seem to have matched the opening quote char
-               // Mark the end of the quotes as our endpoint, so
-               // that we can continue parsing after the current
-               // text
-               //
-               *q = '\0';
-               //
-               // If a '#' is present in a quoted URL,
-               //  treat that as the end of the URL, but we skip
-               //  past the quote to parse the rest of the anchor.
-               //
-               if ((t = strchr(position, '#')) != NULL)
-                   *t = '\0';
-	    }
-	    else
-	    {
-		q = position;
-		while (*q && *q != '>' && !isspace(*q))
-		    q++;
-	    *q = '\0';
-	    }
-	    retriever.got_image(transSGML(position));
+	      }
+	    if (attrs["src"])
+	      {
+		retriever.got_image(transSGML(attrs["src"]));
+	      }
 	    break;
-	}
+	  }
 
 	case 19:	// "li"
-	    if (doindex && !in_title && head.length() < max_head_length)
+	    if (!noindex && !in_title && head.length() < max_head_length)
 		head << "* ";
 	    break;
 
 	case 20:	// "meta"
-	{	    position += length;
-	    Configuration	conf;
-	    conf.NameValueSeparators("=");
-	    conf.Add(position);
-
+	{
 	    //
 	    // First test for old-style meta tags (these break any
 	    // reasonable DTD...)
 	    //
-	    if (conf["htdig-noindex"])
+	    if (attrs["htdig-noindex"])
 	      {
 		retriever.got_noindex();
-		doindex = 0;
-		dofollow = 0;
+		noindex |= TAGmeta_htdig_noindex;
+		nofollow |= TAGmeta_htdig_noindex;
 	      }
-	    if (conf["htdig-index"])
+	    if (attrs["htdig-index"])
 	      {
-		doindex = 1;
-		dofollow = 1;
+		noindex &= ~TAGmeta_htdig_noindex;
+		nofollow &= ~TAGmeta_htdig_noindex;
 	      }
-	    if (conf["htdig-email"])
-	    {
-		retriever.got_meta_email(transSGML(conf["htdig-email"]));
-	    }
-	    if (conf["htdig-notification-date"])
-	    {
-		retriever.got_meta_notification(transSGML(conf["htdig-notification-date"]));
-	    }
-	    if (conf["htdig-email-subject"])
-	    {
-		retriever.got_meta_subject(transSGML(conf["htdig-email-subject"]));
-	    }
-	    if (conf["htdig-keywords"] || conf["keywords"])
+	    if (attrs["htdig-email"])
+	      retriever.got_meta_email(transSGML(attrs["htdig-email"]));
+
+	    if (attrs["htdig-notification-date"])
+	      retriever.got_meta_notification(transSGML(attrs["htdig-notification-date"]));
+
+	    if (attrs["htdig-email-subject"])
+	      retriever.got_meta_subject(transSGML(attrs["htdig-email-subject"]));
+
+	    if (attrs["htdig-keywords"] || attrs["keywords"])
 	    {
 		//
 		// Keywords are added as being at the very top of the
@@ -792,11 +694,11 @@ HTML::do_tag(Retriever &retriever, String &tag)
 		// keywords-factor which is assigned to slot 10 in the
 		// factor table.
 		//
-		char	*keywords = conf["htdig-keywords"];
+		char	*keywords = attrs["htdig-keywords"];
 		if (!keywords)
-		    keywords = conf["keywords"];
+		    keywords = attrs["keywords"];
 		char	*w = HtWordToken(transSGML(keywords));
-		while (w && doindex)
+		while (w && !noindex)
 		{
 		    if (strlen(w) >= minimumWordLength
 				&& ++keywordsCount <= max_keywords)
@@ -806,27 +708,29 @@ HTML::do_tag(Retriever &retriever, String &tag)
 		w = '\0';
 	    }
 	
-	    if (conf["http-equiv"])
+	    if (attrs["http-equiv"])
 	      {
 
 		// <META HTTP-EQUIV=REFRESH case
-		if (mystrcasecmp(conf["http-equiv"], "refresh") == 0
-		    && conf["content"])
+		if (mystrcasecmp(attrs["http-equiv"], "refresh") == 0
+		    && attrs["content"])
 		  {
-		    char *content = conf["content"];
-		    char *q = mystrcasestr(content, "url=");
+		    char *content = attrs["content"];
+		    char *q = mystrcasestr(content, "url");
 		    if (q && *q)
 		      {
-			q += 4; // skiping "URL="
+			q += 3; // skiping "URL"
+			while (*q && ((*q == '=') || isspace(*q))) q++;
 			char *qq = q;
 			while (*qq && (*qq != ';') && (*qq != '"') &&
 			       !isspace(*qq))qq++;
 			*qq = 0;
-			URL *href = new URL(transSGML(q), *base);
+			if (href)
+			  delete href;
+			href = new URL(transSGML(q), *base);
 			// I don't know why anyone would do this, but hey...
-			if (dofollow)
+			if (!nofollow)
 			  retriever.got_href(*href, "");
-			delete href;
 		      }
 		  }
 	      }
@@ -836,25 +740,27 @@ HTML::do_tag(Retriever &retriever, String &tag)
 	    // fly with any reasonable DTD out there
 	    //
 
-	    if (conf["name"] && conf["content"])
+	    if (attrs["name"] && attrs["content"])
 	    {
-		char	*cache = conf["name"];
-
-		which = -1; // What does it do?
+		char	*cache = attrs["name"];
 
 		  // First of all, check for META description
 
-		  if (mystrcasecmp(cache, "description") == 0 
-			 && strlen(conf["content"]) != 0)
-		  {
-		    //
-		    // We need to do two things. First grab the description
-		    //
-		    meta_dsc = transSGML(conf["content"]);
+		if (descriptionMatch.CompareWord(cache)
+			 && strlen(attrs["content"]) != 0)
+		{
+		   //
+		   // We need to do two things. First grab the description
+		   // and clean it up
+		   //
+		   meta_dsc = transSGML(attrs["content"]);
+		   meta_dsc.replace('\n', ' ');
+		   meta_dsc.replace('\r', ' ');
+		   meta_dsc.replace('\t', ' ');
 		   if (meta_dsc.length() > max_meta_description_length)
 		     meta_dsc = meta_dsc.sub(0, max_meta_description_length).get();
 		   if (debug > 1)
-		     cout << "META Description: " << conf["content"] << endl;
+		     cout << "META Description: " << attrs["content"] << endl;
 		   retriever.got_meta_dsc(meta_dsc);
 
 
@@ -863,9 +769,9 @@ HTML::do_tag(Retriever &retriever, String &tag)
 		   // (slot 11 is the new slot for this)
 		   //
 
-		   char        *words = HtWordToken(transSGML(conf["content"]));
+		   char        *words = HtWordToken(transSGML(attrs["content"]));
 		   char        *w = words;
-                   while (w && doindex)
+                   while (w && !noindex)
 		     {
 			if (strlen(w) >= minimumWordLength)
 			  retriever.got_word(w,
@@ -874,13 +780,13 @@ HTML::do_tag(Retriever &retriever, String &tag)
 				 11);
 			w = HtWordToken(0);
 		     }
-		 w = '\0';
+		   w = '\0';
 		}
 
-		if (keywordsMatch.CompareWord(cache))
+		if (keywordsMatch.CompareWord(cache) && !noindex)
 		{
-		    char	*w = HtWordToken(transSGML(conf["content"]));
-		    while (w && doindex)
+		    char	*w = HtWordToken(transSGML(attrs["content"]));
+		    while (w)
 		    {
 			if (strlen(w) >= minimumWordLength
 				&& ++keywordsCount <= max_keywords)
@@ -891,252 +797,123 @@ HTML::do_tag(Retriever &retriever, String &tag)
 		}
 		else if (mystrcasecmp(cache, "htdig-email") == 0)
 		{
-		    retriever.got_meta_email(transSGML(conf["content"]));
+		    retriever.got_meta_email(transSGML(attrs["content"]));
+		}
+		else if (metadatetags.CompareWord(cache) &&
+				config.Boolean("use_doc_date",0))
+		{
+		    retriever.got_time(transSGML(attrs["content"]));
 		}
 		else if (mystrcasecmp(cache, "htdig-notification-date") == 0)
 		{
-		    retriever.got_meta_notification(transSGML(conf["content"]));
+		    retriever.got_meta_notification(transSGML(attrs["content"]));
 		}
 		else if (mystrcasecmp(cache, "htdig-email-subject") == 0)
 		{
-		    retriever.got_meta_subject(transSGML(conf["content"]));
+		    retriever.got_meta_subject(transSGML(attrs["content"]));
 		}
 		else if (mystrcasecmp(cache, "htdig-noindex") == 0)
 		  {
 		    retriever.got_noindex();
-		    doindex = 0;
-		    dofollow = 0;
+		    noindex |= TAGmeta_htdig_noindex;
+		    nofollow |= TAGmeta_htdig_noindex;
 		  }
 		else if (mystrcasecmp(cache, "robots") == 0
-			 && strlen(conf["content"]) !=0)
+			 && strlen(attrs["content"]) !=0)
 		  {
-		    String   content_cache = conf["content"];
-
+		    String   content_cache = attrs["content"];
+		    content_cache.lowercase();
 		    if (content_cache.indexOf("noindex") != -1)
 		      {
-			doindex = 0;
+			noindex |= TAGmeta_robots;
 			retriever.got_noindex();
 		      }
 		    if (content_cache.indexOf("nofollow") != -1)
-		      dofollow = 0;
+			nofollow |= TAGmeta_robots;
 		    if (content_cache.indexOf("none") != -1)
 		      {
-			doindex = 0;
-			dofollow = 0;
+			noindex |= TAGmeta_robots;
+			nofollow |= TAGmeta_robots;
 			retriever.got_noindex();
 		      }
 		  }
 	    }
-	    else if (conf["name"] &&
-		     mystrcasecmp(conf["name"], "htdig-noindex") == 0)
+	    else if (attrs["name"] &&
+		     mystrcasecmp(attrs["name"], "htdig-noindex") == 0)
 	    {
 	        retriever.got_noindex();
-	        doindex = 0;
-		dofollow = 0;
+		noindex |= TAGmeta_htdig_noindex;
+		nofollow |= TAGmeta_htdig_noindex;
 	    }
 	    break;
 	}
 
 	case 21:	// frame
-	case 24:	// embed
-	case 25:	// object
+        case 24:	// embed
 	{
-	    which = -1;
-	    int pos = srcMatch.FindFirstWord(position, which, length);
-	    position += pos + length;
-	    switch (which)
+	  if (attrs["src"])
 	    {
-		case 0:		// "src"
+	      //
+	      // src seen
+	      //
+	      if (!nofollow)
 		{
-		    //
-		    // src seen
-		    //
-		    while (*position && *position != '=')
-			position++;
-		    if (!*position)
-			return;
-		    position++;
-		    while (isspace(*position))
-			position++;
-                   //
-                   // Allow either single quotes or double quotes
-                   // around the URL itself
-                   //
-                   if (*position == '"'||*position == '\'')
-		    {
-			position++;
-			q = strchr(position, position[-1]);
-			if (!q)
-			    break;
-                       //
-                       // We seem to have matched the opening quote char
-                       // Mark the end of the quotes as our endpoint, so
-                       // that we can continue parsing after the current
-                       // text
-                       //
-                       *q = '\0';
-                       //
-                       // If a '#' is present in a quoted URL,
-                       //  treat that as the end of the URL, but we skip
-                       //  past the quote to parse the rest of the anchor.
-                       //
-                       if ((t = strchr(position, '#')) != NULL)
-                           *t = '\0';
-		    }
-		    else
-		    {
-			q = position;
-			while (*q &&
-			       *q != '>' &&
-			       !isspace(*q) && //  *q != '?'   ??? -grh
-			       *q != '#')
-			    q++;
-			*q = '\0';
-		    }
+		  if (href)
 		    delete href;
-		    href = new URL(transSGML(position), *base);
-		    if (dofollow)
-		    {
-			description = 0;
-			retriever.got_href(*href, description);
-			in_ref = 0;
-		    }
-		    break;
+		  href = new URL(transSGML(attrs["src"]), *base);
+		  retriever.got_href(*href, transSGML(attrs["title"]));
+		  in_ref = 0;
 		}
-		break;
 	    }
-	    break;
+	  break;
 	}
-	
+	  
+        case 25:	// object
+	{
+	  if (attrs["data"])
+	    {
+	      //
+	      // data seen
+	      //
+	      if (!nofollow)
+		{
+		  if (href)
+		    delete href;
+		  href = new URL(transSGML(attrs["data"]), *base);
+		  retriever.got_href(*href, transSGML(attrs["title"]));
+		  in_ref = 0;
+		}
+	    }
+	  break;
+	}
+	  
 	case 22:	// area
-	case 26:	// link
+        case 26:	// link
 	{
-	    which = -1;
-	    int pos = hrefMatch.FindFirstWord(position, which, length);
-	    position += pos + length;
-	    switch (which)
+	  if (attrs["href"])
 	    {
-		case 0:		// "href"
+	      // href seen
+	      if (!nofollow)
 		{
-		    //
-		    // href seen
-		    //
-		    while (*position && *position != '=')
-			position++;
-		    if (!*position)
-			return;
-		    position++;
-		    while (isspace(*position))
-			position++;
-                   //
-                   // Allow either single quotes or double quotes
-                   // around the URL itself
-                   //
-                   if (*position == '"'||*position == '\'')
-		    {
-			position++;
-			q = strchr(position, position[-1]);
-			if (!q)
-			    break;
-                       //
-                       // We seem to have matched the opening quote char
-                       // Mark the end of the quotes as our endpoint, so
-                       // that we can continue parsing after the current
-                       // text
-                       //
-                       *q = '\0';
-                       //
-                       // If a '#' is present in a quoted URL,
-                       //  treat that as the end of the URL, but we skip
-                       //  past the quote to parse the rest of the anchor.
-                       if ((t = strchr(position, '#')) != NULL)
-                           *t = '\0';
-		    }
-		    else
-		    {
-			q = position;
-			while (*q &&
-			       *q != '>' &&
-			       !isspace(*q) && //  *q != '?'   ???? --grh
-			       *q != '#')
-			    q++;
-			*q = '\0';
-		    }
+		  if (href)
 		    delete href;
-		    href = new URL(transSGML(position), *base);
-		    if (dofollow)
-		    {
-			description = 0;
-			retriever.got_href(*href, description);
-			in_ref = 0;
-		    }
-		    break;
+		  href = new URL(transSGML(attrs["href"]), *base);
+		  retriever.got_href(*href, transSGML(attrs["title"]));
+		  in_ref = 0;
 		}
-
-		default:
-		    break;
 	    }
-	    break;
+	  break;
 	}
-
+	  
 	case 23:	// base
 	{
-	    which = -1;
-	    int pos = hrefMatch.FindFirstWord(position, which, length);
-	    position += pos + length;
-	    switch (which)
+	  if (attrs["href"])
 	    {
-		case 0:		// "href"
-		{
-		    while (*position && *position != '=')
-			position++;
-		    if (!*position)
-			return;
-		    position++;
-		    while (isspace(*position))
-			position++;
-                   //
-                   // Allow either single quotes or double quotes
-                   // around the URL itself
-                   //
-                   if (*position == '"'||*position == '\'')
-		    {
-			position++;
-			q = strchr(position, position[-1]);
-			if (!q)
-			    break;
-                       //
-                       // We seem to have matched the opening quote char
-                       // Mark the end of the quotes as our endpoint, so
-                       // that we can continue parsing after the current
-                       // text
-                       //
-                       *q = '\0';
-                       //
-                       // If a '#' is present in a quoted URL,
-                       //  treat that as the end of the URL, but we skip
-                       //  past the quote to parse the rest of the anchor.
-                       //
-                       // Is there a better way of looking for these?
-                       //
-                       if ((t = strchr(position, '#')) != NULL)
-                           *t = '\0';
-		    }
-		    else
-		    {
-			q = position;
-			while (*q &&
-			       *q != '>' &&
-			       !isspace(*q) && // *q != '?'   ??? -grh
-			       *q != '#')
-			    q++;
-		    *q = '\0';
-		    }
-		    URL tempBase(transSGML(position), *base);
-		    *base = tempBase;
-		}
+	      //URL tempBase(transSGML(attrs["href"]), *base);
+	      URL tempBase(transSGML(attrs["href"]));
+	      *base = tempBase;
 	    }
-	    break;
+	  break;
 	}
 	
 	default:
@@ -1155,7 +932,7 @@ HTML::transSGML(char *str)
     unsigned char	*text = (unsigned char *)str;
 
     convert = 0;
-    while (*text)
+    while (text && *text)
     {
 	if (*text == '&')
 	{
