@@ -3,7 +3,7 @@
 //
 // Implementation of Retriever
 //
-// $Id: Retriever.cc,v 1.36.2.20 2000/02/15 23:10:36 grdetil Exp $
+// $Id: Retriever.cc,v 1.36.2.28 2002/01/25 04:44:33 ghutchis Exp $
 //
 
 #include "Retriever.h"
@@ -334,6 +334,7 @@ Retriever::parse_url(URLRef &urlRef)
     static int	index = 0;
     Server		*server;
     static int		local_urls_only = config.Boolean("local_urls_only");
+    static int		mark_dead_servers = config.Boolean("ignore_dead_servers");
 
 //	cout << "**** urlRef URL = '" << urlRef.URL() << "', referer = '" <<
 //		urlRef.Referer() << "'\n";
@@ -504,7 +505,7 @@ Retriever::parse_url(URLRef &urlRef)
 			   urlRef.Referer(),
 			   Document::Document_no_host);
 	    words.MarkGone();
-	    if (server)
+	    if (server && mark_dead_servers)
 		server->IsDead(1);
 	    break;
 
@@ -516,7 +517,7 @@ Retriever::parse_url(URLRef &urlRef)
 			   urlRef.Referer(),
 			   Document::Document_no_server);
 	    words.MarkGone();
-	    if (server)
+	    if (server && mark_dead_servers)
 		server->IsDead(1);
 	    break;
 
@@ -561,6 +562,7 @@ Retriever::RetrievedDocument(Document &doc, char *, DocumentRef *ref)
     current_ref = ref;
     current_anchor_number = 0;
     current_title = 0;
+    current_time = 0;
     current_head = 0;
     current_meta_dsc = 0;
 
@@ -571,7 +573,13 @@ Retriever::RetrievedDocument(Document &doc, char *, DocumentRef *ref)
     // This will generate the Parsable object as a specific parser
     //
     Parsable	*parsable = doc.getParsable();
-    parsable->parse(*this, *base);
+    if (parsable)
+      parsable->parse(*this, *base);
+    else
+      { // If we didn't get a parser, then we should get rid of this!
+	ref->DocState(Reference_noindex);
+	return;
+      }
 
     //
     // We don't need to dispose of the parsable object since it will
@@ -583,7 +591,10 @@ Retriever::RetrievedDocument(Document &doc, char *, DocumentRef *ref)
     //
     ref->DocHead(current_head);
     ref->DocMetaDsc(current_meta_dsc);
-    ref->DocTime(doc.ModTime());
+    if (current_time == 0)
+      ref->DocTime(doc.ModTime());
+    else
+      ref->DocTime(current_time);
     ref->DocTitle(current_title);
     ref->DocSize(doc.Length());
     ref->DocAccessed(time(0));
@@ -823,7 +834,7 @@ Retriever::GetLocal(char *url)
     if (strstr(url, ".."))
         return 0;
     
-    String *prefix, *path;
+    String *prefix, *path, *defaultdoc;
     StringList *local_names = new StringList();
     prefixes->Start_Get();
     paths->Start_Get();
@@ -837,7 +848,7 @@ Retriever::GetLocal(char *url)
 	    *local += &url[prefix->length()];
 	    if (local->last() == '/' && defaultdocs) {
 	      defaultdocs->Start_Get();
-	      while (String *defaultdoc = (String *)defaultdocs->Get_Next()) {
+	      while (defaultdoc = (String *)defaultdocs->Get_Next()) {
 		String *localdefault = new String(*local, local->length()+defaultdoc->length()+1);
 		localdefault->append(*defaultdoc);
 		local_names->Add(localdefault);
@@ -868,6 +879,7 @@ Retriever::GetLocalUser(char *url, StringList *defaultdocs)
 {
     static StringList *prefixes = 0, *paths = 0, *dirs = 0;
     static Dictionary home_cache;
+    String *defaultdoc;
 
     //
     // Initialize prefix/path list if this is the first time.
@@ -964,7 +976,7 @@ Retriever::GetLocalUser(char *url, StringList *defaultdocs)
 	*local += rest;
 	if (local->last() == '/' && defaultdocs) {
 	  defaultdocs->Start_Get();
-	  while (String *defaultdoc = (String *)defaultdocs->Get_Next()) {
+	  while (defaultdoc = (String *)defaultdocs->Get_Next()) {
 	    String *localdefault = new String(*local, local->length()+defaultdoc->length()+1);
 	    localdefault->append(*defaultdoc);
 	    local_names->Add(localdefault);
@@ -1099,6 +1111,122 @@ Retriever::got_title(char *title)
 }
 
 
+#define EPOCH	1970
+
+//
+// time_t parsedcdate(char *date)
+//   - converts ISO 8601 (YYYY-MM-DD) date string into a time value
+//
+time_t
+parsedcdate(char *date)
+{
+    char	*s;
+    int		day, month, year, hour, minute, second;
+
+    s = date;
+    while (isspace(*s))
+	s++;
+
+    // get year...
+    if (!isdigit(*s))
+	return 0;
+    year = 0;
+    while (isdigit(*s))
+	year = year * 10 + (*s++ - '0');
+    if (year < 69)
+	year += 2000;
+    else if (year < 1900)
+	year += 1900;
+    else if (year >= 19100)	// seen some programs do it, why not check?
+	year -= (19100-2000);
+    while (isspace(*s))
+	s++;
+
+    // get month...
+    if (!isdigit(*s))
+	return 0;
+    month = 0;
+    while (isdigit(*s))
+	month = month * 10 + (*s++ - '0');
+    if (month < 1 || month > 12)
+	return 0;
+    while (*s == '-' || isspace(*s))
+	s++;
+
+    // get day...
+    if (!isdigit(*s))
+	return 0;
+    day = 0;
+    while (isdigit(*s))
+	day = day * 10 + (*s++ - '0');
+    if (day < 1 || day > 31)
+	return 0;
+    while (*s == '-' || isspace(*s))
+	s++;
+
+    // optionally get hour...
+    hour = 0;
+    while (isdigit(*s))
+	hour = hour * 10 + (*s++ - '0');
+    if (hour > 23)
+	return 0;
+    while (*s == ':' || isspace(*s))
+	s++;
+
+    // optionally get minute...
+    minute = 0;
+    while (isdigit(*s))
+	minute = minute * 10 + (*s++ - '0');
+    if (minute > 59)
+	return 0;
+    while (*s == ':' || isspace(*s))
+	s++;
+
+    // optionally get second...
+    second = 0;
+    while (isdigit(*s))
+	second = second * 10 + (*s++ - '0');
+    if (second > 59)
+	return 0;
+    while (*s == ':' || isspace(*s))
+	s++;
+
+    //
+    // Calculate date as seconds since 01 Jan 1970 00:00:00 GMT
+    // This is based somewhat on the date calculation code in NetBSD's
+    // cd9660_node.c code, for which I was unable to find a reference.
+    // It works, though!
+    //
+    return (time_t) (((((367L*year - 7L*(year+(month+9)/12)/4
+				   - 3L*(((year)+((month)+9)/12-1)/100+1)/4
+				   + 275L*(month)/9 + day) -
+			(367L*EPOCH - 7L*(EPOCH+(1+9)/12)/4
+				   - 3L*((EPOCH+(1+9)/12-1)/100+1)/4
+				   + 275L*1/9 + 1))
+		       * 24 + hour) * 60 + minute) * 60 + second);
+}
+
+//*****************************************************************************
+// void Retriever::got_time(char *time)
+//
+void
+Retriever::got_time(char *time)
+{
+    time_t   new_time;
+    if (debug > 1)
+      cout << "\ntime: " << time << endl;
+    //
+    // As defined by the Dublin Core, this should be YYYY-MM-DD
+    // In the future, we'll need to deal with the scheme portion
+    //  in case someone picks a different format.
+    //
+    new_time = parsedcdate(time);
+    if (new_time)
+	current_time = new_time;
+    // If we can't convert it, current_time stays the same and we get
+    // the default--the date returned by the server...
+}
+
 //*****************************************************************************
 // void Retriever::got_anchor(char *anchor)
 //
@@ -1152,6 +1280,7 @@ Retriever::got_href(URL &url, char *description)
     //
     if (IsValidURL(url.get()))
     {
+	String	oldurl;
 	//
 	// It is valid.  Normalize it (resolve cnames for the server)
 	// and check again...
@@ -1160,9 +1289,16 @@ Retriever::got_href(URL &url, char *description)
 	{
 	    cout << "resolving '" << url.get() << "'\n";
 	    cout.flush();
+	    oldurl = url.get();
 	}
 
 	url.normalize();
+	url.rewrite();
+	if (debug > 2 && strcmp(oldurl.get(), url.get()) != 0)
+	{
+	    cout << "normalized/rewritten as '" << url.get() << "'\n";
+	    cout.flush();
+	}
 
 	// If it is a backlink from the current document,
 	// just update that field.  Writing to the database
@@ -1211,11 +1347,8 @@ Retriever::got_href(URL &url, char *description)
 		return;
 	    }
 
-	    if (ref->DocHopCount() != -1 &&
-		ref->DocHopCount() < currenthopcount + 1)
-	       // If we had taken the path through this ref
-	       // We'd be here faster than currenthopcount
-	       currenthopcount = ref->DocHopCount();  // So update it!
+	    if (ref->DocHopCount() > currenthopcount + 1)
+		ref->DocHopCount(currenthopcount + 1);
 
 	    docs.Add(*ref);
 
@@ -1287,7 +1420,9 @@ Retriever::got_href(URL &url, char *description)
 void
 Retriever::got_redirect(char *new_url, DocumentRef *old_ref)
 {
-    URL	url(new_url);
+    // First we must piece together the new URL, which may be relative
+    URL parent(old_ref->DocURL());
+    URL	url(new_url, parent);
 
     if (debug > 2)
 	cout << "redirect: " << url.get() << endl;
@@ -1302,6 +1437,7 @@ Retriever::got_redirect(char *new_url, DocumentRef *old_ref)
     //
     if (IsValidURL(url.get()))
     {
+	String	oldurl;
 	//
 	// It is valid.  Normalize it (resolve cnames for the server)
 	// and check again...
@@ -1310,9 +1446,17 @@ Retriever::got_redirect(char *new_url, DocumentRef *old_ref)
 	{
 	    cout << "resolving '" << url.get() << "'\n";
 	    cout.flush();
+	    oldurl = url.get();
 	}
 
 	url.normalize();
+	url.rewrite();
+	if (debug > 2 && strcmp(oldurl.get(), url.get()) != 0)
+	{
+	    cout << "normalized/rewritten as '" << url.get() << "'\n";
+	    cout.flush();
+	}
+
 	if (limitsn.FindFirst(url.get()) >= 0)
 	{
 	    //
