@@ -5,7 +5,7 @@
 //
 //
 #if RELEASE
-static char RCSid[] = "$Id: EndingsDB.cc,v 1.3.2.1 1999/03/29 15:59:43 ghutchis Exp $";
+static char RCSid[] = "$Id: EndingsDB.cc,v 1.3.2.3 2002/01/26 03:43:38 ghutchis Exp $";
 #endif
 
 #include "Endings.h"
@@ -17,7 +17,18 @@ static char RCSid[] = "$Id: EndingsDB.cc,v 1.3.2.1 1999/03/29 15:59:43 ghutchis
 #include <stdio.h>
 #include <fstream.h>
 #include <stdlib.h>
+
+// This is an attempt to get around compatibility problems
+// with the included regex
+#ifdef USE_RX
+#include <rxposix.h>
+#else // Use regex
+#ifdef HAVE_BROKEN_REGEX
 #include <regex.h>
+#else // included regex code and header
+#include "gregex.h"
+#endif
+#endif
 
 
 //*****************************************************************************
@@ -189,7 +200,14 @@ Endings::createRoot(Dictionary &rules, char *word2root, char *root2word, char *d
 	//
 	for (int i = 0; i < wordList.Count(); i++)
 	{
-	    w2r->Put(((String *)wordList[i])->get(), word, strlen(input));
+	    //
+	    // Append to existing record if there is one.
+	    //
+	    data = "";
+	    if (w2r->Get(((String *)wordList[i])->get(), data) == OK)
+		data << ' ';
+	    data << word;
+	    w2r->Put(((String *)wordList[i])->get(), data);
 	}
     }
 
