@@ -1,17 +1,23 @@
 //
 // Implementation of String class
 //
-// $Id: String.cc,v 1.16.2.3 1999/11/26 21:59:26 grdetil Exp $
+// $Id: String.cc,v 1.16.2.6 2001/11/20 23:15:32 grdetil Exp $
+//
+// Part of the ht://Dig package   <http://www.htdig.org/>
+// Copyright (c) 1995-2001 The ht://Dig Group
+// For copyright details, see the file COPYING in your distribution
+// or the GNU Public License version 2 or later
+// <http://www.gnu.org/copyleft/gpl.html>
 //
 #if RELEASE
-static char	RCSid[] = "$Id: String.cc,v 1.16.2.3 1999/11/26 21:59:26 grdetil Exp $";
+static char	RCSid[] = "$Id: String.cc,v 1.16.2.6 2001/11/20 23:15:32 grdetil Exp $";
 #endif
 
 
 #include "htString.h"
 
 #include <unistd.h>
-#include <stream.h>
+#include <fstream.h>
 #include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -55,7 +61,7 @@ String::String(char *s, int len)
 {
     Allocated = 0;
     Length = 0;
-    if (s && len != 0)
+    if (s && len > 0)
 	copy(s, len, len);
 }
 
@@ -91,9 +97,16 @@ String::~String()
 
 void String::operator = (const String &s)
 {
-    allocate_space(s.length());
-    Length = s.length();
-    copy_data_from(s.Data, Length);
+    if (s.length() > 0) 
+    {
+	allocate_space(s.length());
+	Length = s.length();
+	copy_data_from(s.Data, Length);
+    }
+    else
+    {
+	Length = 0;
+    }
 }
 
 void String::operator = (char *s)
@@ -130,7 +143,7 @@ void String::append(char *s)
 
 void String::append(char *s, int slen)
 {
-    if (!s || !slen)
+    if (!s || slen <= 0)
 	return;
 
 //    if ( slen == 1 ) 
@@ -245,7 +258,7 @@ int String::as_integer(int def)
 
 String String::sub(int start, int len) const
 {
-    if (start > Length)
+    if (start > Length || start < 0 || len < 0)
 	return 0;
 
     if (len > Length - start)
@@ -622,3 +635,38 @@ void String::debug(ostream &o)
 }
 
 
+int String::readLine(FILE *in)
+{
+    Length = 0;
+    allocate_fix_space(2048);
+
+    while (fgets(Data + Length, Allocated - Length, in))
+    {
+	Length += strlen(Data + Length);
+	if (Length == 0)
+	    continue;
+	if (Data[Length - 1] == '\n')
+	{
+	    //
+	    // A full line has been read.  Return it.
+	    //
+	    chop('\n');
+	    return 1;
+	}
+	if (Allocated > Length + 1)
+	{
+	    //
+	    // Not all available space filled. Probably EOF?
+	    //
+	    continue;
+	}
+	//
+	// Only a partial line was read. Increase available space in 
+	// string and read some more.
+	//
+	reallocate_space(Allocated << 1);
+    }
+    chop('\n');
+
+    return Length > 0;
+}
