@@ -5,7 +5,7 @@
 //
 //
 #if RELEASE
-static char RCSid[] = "$Id: URL.cc,v 1.18.2.7 2000/02/22 18:40:33 grdetil Exp $";
+static char RCSid[] = "$Id: URL.cc,v 1.18.2.10 2001/10/12 20:04:30 grdetil Exp $";
 #endif
 
 #include "URL.h"
@@ -13,6 +13,7 @@ static char RCSid[] = "$Id: URL.cc,v 1.18.2.7 2000/02/22 18:40:33 grdetil Exp $"
 #include "Configuration.h"
 #include "StringMatch.h"
 #include "StringList.h"
+#include "HtURLRewriter.h"
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -202,12 +203,12 @@ URL::URL(char *ref, URL &parent)
 		    //
 		}
 	    }
-
-            //
-            // Get rid of loop-causing constructs in the path
-            //
-            normalizePath();
 	}
+
+        //
+        // Get rid of loop-causing constructs in the path
+        //
+        normalizePath();
     }
 	
     //
@@ -222,6 +223,13 @@ URL::URL(char *ref, URL &parent)
     _url << _path;
 }
 
+void URL::rewrite()
+{
+      String _old = _url.get();
+      HtURLRewriter::instance()->Replace(_url);
+      if (strcmp(_old.get(), _url.get()) != 0)
+              parse(_url.get());
+}
 
 //*****************************************************************************
 // void URL::parse(char *u)
@@ -469,8 +477,10 @@ void URL::removeIndex(String &path)
       defaultdoc->Pattern(l.Join('|'));
       l.Release();
     }
+    int which, length;
     if (defaultdoc->hasPattern() &&
-            defaultdoc->CompareWord(path.sub(filename)))
+	    defaultdoc->CompareWord(path.sub(filename), which, length) &&
+	    filename+length == path.length())
 	path.chop(path.length() - filename);
 }
 
