@@ -1,15 +1,26 @@
 //
 // Just Another String class.
 //
-// $Id: htString.h,v 1.5 1999/02/01 04:02:25 hp Exp $
+// $Id: htString.h,v 1.5.2.4 2001/12/20 07:17:25 ghutchis Exp $
+//
+// Part of the ht://Dig package   <http://www.htdig.org/>
+// Copyright (c) 1995-2001 The ht://Dig Group
+// For copyright details, see the file COPYING in your distribution
+// or the GNU Public License version 2 or later
+// <http://www.gnu.org/copyleft/gpl.html>
 //
 #ifndef __String_h
 #define __String_h
 
 #include "Object.h"
 #include <stdarg.h>
-
-class ostream;
+#include <stdio.h>
+#ifdef HAVE_OSTREAM_H
+#include <ostream.h>
+#endif
+#ifdef HAVE_IOSTREAM_H
+#include <iostream.h>
+#endif
 
 class String : public Object
 {
@@ -72,6 +83,7 @@ public:
     // Access to specific characters
     //
     char		&operator [] (int n);
+    char		Nth(int n) {return (*this)[n];}
     char		last();
 
     //
@@ -139,6 +151,8 @@ public:
 
     friend ostream	&operator << (ostream &o, String &s);
 
+    int			readLine(FILE *in);
+
     void		lowercase();
     void		uppercase();
 
