@@ -3,7 +3,7 @@
 //
 // Indexing the "doc_db" database by id-number in "doc_index".
 //
-// $Id: docs.cc,v 1.14.2.2 1999/03/22 23:39:30 grdetil Exp $
+// $Id: docs.cc,v 1.14.2.4 2002/01/19 00:25:13 ghutchis Exp $
 //
 //
 
@@ -42,6 +42,10 @@ convertDocs(char *doc_db, char *doc_index)
     //
     db.Open(doc_db);
     urls = db.URLs();
+    if (urls->Count() == 0)
+      {
+	reportError("Document database has no URLs. Check your config file and try running htdig again.");
+      }
 	
     urls->Start_Get();
     String		*url;
@@ -106,7 +110,7 @@ convertDocs(char *doc_db, char *doc_index)
     if (stats)
       {
 	cout << "htmerge: Total documents: " << document_count << endl;
-	cout << "htmerge: Total doc db size (in K): ";
+	cout << "htmerge: Total size of documents (in K): ";
 	cout << docdb_size / 1024 << endl;
       }
 
