@@ -6,7 +6,7 @@
 //
 //
 #if RELEASE
-static char RCSid[] = "$Id: Display.cc,v 1.54.2.27 2000/02/17 16:46:25 grdetil Exp $";
+static char RCSid[] = "$Id: Display.cc,v 1.54.2.49 2002/01/16 23:00:17 grdetil Exp $";
 #endif
 
 #include "htsearch.h"
@@ -22,8 +22,11 @@ static char RCSid[] = "$Id: Display.cc,v 1.54.2.27 2000/02/17 16:46:25 grdetil E
 #include <syslog.h>
 #include <locale.h>
 #include "HtURLCodec.h"
+#include "HtURLRewriter.h"
 #include "HtWordType.h"
 
+extern int		debug;
+
 //*****************************************************************************
 //
 Display::Display(char *indexFile, char *docFile)
@@ -106,6 +109,7 @@ Display::display(int pageNumber)
         logSearch(pageNumber, matches);
     }
 
+    displayHTTPheaders();
     setVariables(pageNumber, matches);
 	
     //
@@ -120,15 +124,11 @@ Display::display(int pageNumber)
 	// No matches.
 	//
         delete matches;
-	if (config.Boolean("nph")) cout << "HTTP/1.0 200 OK\r\n";
-	cout << "Content-type: text/html\r\n\r\n";
 	displayNomatch();
 	return;
     }
     // maxScore = match->getScore();	// now done in buildMatchList()
     	
-    if (config.Boolean("nph")) cout << "HTTP/1.0 200 OK\r\n";
-    cout << "Content-type: text/html\r\n\r\n";
     String	wrap_file = config["search_results_wrapper"];
     String	*wrapper = 0;
     char	*header = 0, *footer = 0;
@@ -179,7 +179,7 @@ Display::display(int pageNumber)
     {
 	if (currentMatch >= startAt)
 	{
-	    match->setRef(docDB[match->getURL()]);
+	    match->setRef(docDB.FindCoded(match->getURL()));
 	    DocumentRef	*ref = match->getRef();
 	    if (!ref)
 		continue;	// The document isn't present for some reason
@@ -235,9 +235,13 @@ Display::displayMatch(ResultMatch *match, int current)
 	
     DocumentRef	*ref = match->getRef();
 
-    char    *url = match->getURL();
-    vars.Add("URL", new String(url));
+    char    *coded_url = match->getURL();
+    String url = HtURLCodec::instance()->decode(coded_url);
+    HtURLRewriter::instance()->Replace(url);
+    ref->DocURL(url.get());	// for star_patterns & template_patterns match
+    vars.Add("URL", new String(url.get()));
     
+    vars.Remove("ANCHOR");	// get rid of any previous setting
     int     iA = ref->DocAnchor();
     
     String  *anchor = 0;
@@ -273,6 +277,7 @@ Display::displayMatch(ResultMatch *match, int current)
 	vars.Remove("ANCHOR");
       }
     
+    vars.Add("METADESCRIPTION", new String(ref->DocMetaDsc()));
     vars.Add("SCORE", new String(form("%d", match->getScore())));
     vars.Add("CURRENT", new String(form("%d", current)));
     char	*title = ref->DocTitle();
@@ -402,7 +407,7 @@ Display::setVariables(int pageNumber, List *matches)
     else if (mystrcasecmp(config["match_method"], "or") == 0)
 	vars.Add("MATCH_MESSAGE", new String("some"));
     vars.Add("MATCHES", new String(form("%d", nMatches)));
-    vars.Add("PLURAL_MATCHES", new String(nMatches == 1 ? (char *)"" : (char *)"s"));
+    vars.Add("PLURAL_MATCHES", new String((nMatches == 1) ? (char *)"" : config["plural_suffix"]));
     vars.Add("PAGE", new String(form("%d", pageNumber)));
     vars.Add("PAGES", new String(form("%d", nPages)));
     vars.Add("FIRSTDISPLAYED",
@@ -423,6 +428,12 @@ Display::setVariables(int pageNumber, List *matches)
     } else {
       vars.Add("CGI", new String(getenv("SCRIPT_NAME")));
     }
+    vars.Add("STARTYEAR", new String(config["startyear"]));
+    vars.Add("STARTMONTH", new String(config["startmonth"]));
+    vars.Add("STARTDAY", new String(config["startday"]));
+    vars.Add("ENDYEAR", new String(config["endyear"]));
+    vars.Add("ENDMONTH", new String(config["endmonth"]));
+    vars.Add("ENDDAY", new String(config["endday"]));
 	
     String	*str;
     char	*format = input->get("format");
@@ -498,27 +509,96 @@ Display::setVariables(int pageNumber, List *matches)
 	int	ivalue = atoi(builds[b+4]);
 	int	ilabel = atoi(builds[b+5]);
 	int	nsel = 0;
+	int	mult = 0, asinput = 0;
+	char	*cp;
+	char	sepc = '\001';
+	String	currval;
+	String	pre, post;
+	QuotedStringList	nameopt(builds[b], ",", 1);
 	QuotedStringList	namelist(config[builds[b+2]], " \t\r\n");
 	if (ntuple > 0 && ivalue > 0 && ivalue <= ntuple
-	  && ilabel > 0 && ilabel <= ntuple && namelist.Count() % ntuple == 0)
+	  && ilabel > 0 && ilabel <= ntuple && namelist.Count() % ntuple == 0
+	  && nameopt.Count() > 0)
 	{
+	    if (strcmp(builds[b+1], "restrict") == 0
+		|| strcmp(builds[b+1], "exclude") == 0)
+		    sepc = '|';
+	    if (nameopt.Count() == 1)
+		;		// default is single select
+	    else if (mystrcasecmp(nameopt[1], "multiple") == 0)
+		mult = 1;
+	    else if (mystrcasecmp(nameopt[1], "radio") == 0)
+		asinput = 1;
+	    else if (mystrcasecmp(nameopt[1], "checkbox") == 0)
+	    {
+		mult = 1;
+		asinput = 1;
+	    }
+	    if (nameopt.Count() > 2)
+		pre = nameopt[2];
+	    else
+		pre = "";
+	    if (nameopt.Count() > 3)
+		post = nameopt[3];
+	    else
+		post = "";
+
 	    str = new String();
-	    *str << "<select name=\"" << builds[b+1] << "\">\n";
+	    if (!asinput)
+	    {
+		*str << "<select ";
+		if (mult)
+		    *str << "multiple ";
+		*str << "name=\"" << builds[b+1] << "\">\n";
+	    }
 	    for (i = 0; i < namelist.Count(); i += ntuple)
 	    {
-		*str << "<option value=\"" << namelist[i+ivalue-1] << '"';
-		if (mystrcasecmp(namelist[i+ivalue-1],config[builds[b+6]]) == 0)
+		if (*builds[b+6])
+		    currval = config[builds[b+6]];
+		else if (input->exists(builds[b+1]))
+		    currval = input->get(builds[b+1]);
+		else
+		    currval = 0;
+		if (!asinput)
+		    *str << pre << "<option value=\"" << namelist[i+ivalue-1] << '"';
+		else if (mult)
+		    *str << pre << "<input type=\"checkbox\" name=\"" << builds[b+1]
+			 << "\" value=\"" << namelist[i+ivalue-1] << '"';
+		else
+		    *str << pre << "<input type=\"radio\" name=\"" << builds[b+1]
+			 << "\" value=\"" << namelist[i+ivalue-1] << '"';
+		if (!mult && mystrcasecmp(namelist[i+ivalue-1], currval) == 0
+		    || mult &&
+		     (cp = mystrcasestr(currval, namelist[i+ivalue-1])) != NULL
+			&& (cp == currval.get() || cp[-1] == '\001' || cp[-1] == sepc)
+			&& (*(cp += strlen(namelist[i+ivalue-1])) == '\0'
+				|| *cp == '\001' || *cp == sepc))
 		{
-		    *str << " selected";
+		    if (!asinput)
+			*str << " selected";
+		    else
+			*str << " checked";
 		    ++nsel;
 		}
-		*str << '>' << namelist[i+ilabel-1] << '\n';
+		*str << '>' << namelist[i+ilabel-1] << post << '\n';
 	    }
 	    if (!nsel && builds[b+7][0] && input->exists(builds[b+1]))
-		*str << "<option value=\"" << input->get(builds[b+1])
-		     << "\" selected>" << builds[b+7] << '\n';
-	    *str << "</select>\n";
-	    vars.Add(builds[b], str);
+	    {
+		if (!asinput)
+		    *str << pre << "<option value=\"" << input->get(builds[b+1])
+		         << "\" selected>" << builds[b+7] << post << '\n';
+		else if (mult)
+		    *str << pre << "<input type=\"checkbox\" name=\"" << builds[b+1]
+			 << "\" value=\"" << input->get(builds[b+1])
+			 << "\" checked>" << builds[b+7] << post << '\n';
+		else
+		    *str << pre << "<input type=\"radio\" name=\"" << builds[b+1]
+			 << "\" value=\"" << input->get(builds[b+1])
+			 << "\" checked>" << builds[b+7] << post << '\n';
+	    }
+	    if (!asinput)
+		*str << "</select>\n";
+	    vars.Add(nameopt[0], str);
 	}
     }
 	
@@ -558,6 +638,8 @@ Display::setVariables(int pageNumber, List *matches)
 	QuotedStringList	pnt(config["page_number_text"], " \t\r\n");
 	QuotedStringList	npnt(config["no_page_number_text"], " \t\r\n");
 	QuotedStringList	sep(config["page_number_separator"], " \t\r\n");
+	if (nPages > config.Value("maximum_page_buttons", 10))
+	    nPages = config.Value("maximum_page_buttons", 10);
 	for (i = 1; i <= nPages; i++)
 	{
 	    if (i == pageNumber)
@@ -631,6 +713,18 @@ Display::createURL(String &url, int pageNumber)
 	url << "keywords=" << encodeInput("keywords") << ';';
     if (input->exists("words"))
 	url << "words=" << encodeInput("words") << ';';
+    if (input->exists("startyear"))
+	url << "startyear=" << encodeInput("startyear") << ';';
+    if (input->exists("startmonth"))
+	url << "startmonth=" << encodeInput("startmonth") << ';';
+    if (input->exists("startday"))
+	url << "startday=" << encodeInput("startday") << ';';
+    if (input->exists("endyear"))
+	url << "endyear=" << encodeInput("endyear") << ';';
+    if (input->exists("endmonth"))
+	url << "endmonth=" << encodeInput("endmonth") << ';';
+    if (input->exists("endday"))
+	url << "endday=" << encodeInput("endday") << ';';
     StringList form_vars(config["allow_in_form"], " \t\r\n");
     for (i= 0; i < form_vars.Count(); i++)
     {
@@ -645,6 +739,17 @@ Display::createURL(String &url, int pageNumber)
     url << "page=" << pageNumber;
 }
 
+//*****************************************************************************
+void
+Display::displayHTTPheaders()
+{
+    String  content_type = config["search_results_contenttype"];
+    if (config.Boolean("nph"))
+	cout << "HTTP/1.0 200 OK\r\n";
+    if (content_type.length())
+	cout << "Content-type: " << content_type << "\r\n\r\n";
+}
+
 //*****************************************************************************
 void
 Display::displayHeader()
@@ -670,9 +775,7 @@ Display::displayNomatch()
 void
 Display::displaySyntaxError(char *message)
 {
-    if (config.Boolean("nph")) cout << "HTTP/1.0 200 OK\r\n";
-    cout << "Content-type: text/html\r\n\r\n";
-
+    displayHTTPheaders();
     setVariables(0, 0);
     vars.Add("SYNTAXERROR", new String(message));
     displayParsedFile(config["syntax_error_file"]);
@@ -691,6 +794,8 @@ Display::displayParsedFile(char *filename)
     }
     if (fl)
 	fclose(fl);
+    else if (debug)
+	cerr << "displayParsedFile: Can't open " << filename << endl;
 }
 
 //*****************************************************************************
@@ -797,6 +902,8 @@ Display::generateStars(DocumentRef *ref, int right)
     }
     int		nStars = int(score * (maxStars - 1) + 0.5) + 1;
 
+    vars.Add("NSTARS", new String(form("%d", nStars)));
+
     if (right)
     {
 	for (i = 0; i < maxStars - nStars; i++)
@@ -848,6 +955,10 @@ Display::readFile(char *filename)
     {
 	*s << line;
     }
+    if (fl)
+	fclose(fl);
+    else if (debug)
+	cerr << "readFile: Can't open " << filename << endl;
     return s;
 }
 
@@ -905,8 +1016,8 @@ Display::expandVariables(char *str)
 		state = StStart;
 		break;
 	    case StVarStart:
-		if (*str == '%')
-		    var << *str;	// code for URL-encoded variable
+		if (*str == '%' || *str == '=')
+		    var << *str;	// code for URL-encoded/decoded variable
 		else if (*str == '&')
 		{
 		    var << *str;	// code for SGML-encoded variable
@@ -915,7 +1026,7 @@ Display::expandVariables(char *str)
 		}
 		else if (*str == '(' || *str == '{')
 		    state = StVarClose;
-		else if (isalpha(*str) || *str == '_')
+		else if (isalnum(*str) || *str == '_' || *str == '-')
 		{
 		    var << *str;
 		    state = StVarPlain;
@@ -926,13 +1037,13 @@ Display::expandVariables(char *str)
 	    case StVarClose:
 		if (*str == ')' || *str == '}')
 		    state = StGotVar;
-		else if (isalpha(*str) || *str == '_')
+		else if (isalnum(*str) || *str == '_' || *str == '-')
 		    var << *str;
 		else
 		    state = StStart;
 		break;
 	    case StVarPlain:
-		if (isalpha(*str) || *str == '_')
+		if (isalnum(*str) || *str == '_' || *str == '-')
 		    var << *str;
 		else
 		{
@@ -975,7 +1086,7 @@ Display::outputVariable(char *var)
     // see if we can find a good replacement for it, either in our
     // vars dictionary or in the environment variables.
     name = var;
-    while (*name == '&' || *name == '%')
+    while (*name == '&' || *name == '%' || *name == '=')
 	name++;
     temp = (String *) vars[name];
     if (temp)
@@ -990,7 +1101,9 @@ Display::outputVariable(char *var)
     {
 	if (*name == '%')
 	    encodeURL(value);
-	else
+	else if (*name == '=')
+	    decodeURL(value);
+	else // (*name == '&')
 	    encodeSGML(value);
     }
     cout << value;
@@ -1008,6 +1121,243 @@ Display::buildMatchList()
     double      date_factor = config.Double("date_factor");
     SortType	typ = sortType();
 	
+
+    // Additions made here by Mike Grommet 4-1-99 ...
+
+    tm startdate;     // structure to hold the startdate specified by the user
+    tm enddate;       // structure to hold the enddate specified by the user
+    time_t now = time((time_t *)0); 	// fill in all fields for mktime
+    tm *lt = localtime(&now); 		//  - Gilles's fix
+    startdate = *lt; 
+    enddate = *lt; 
+
+    time_t eternity = ~(1<<(sizeof(time_t)*8-1));  // will be the largest value holdable by a time_t
+    tm *endoftime;     // the time_t eternity will be converted into a tm, held by this variable
+
+    time_t timet_startdate;
+    time_t timet_enddate;
+    int monthdays[] = {31,28,31,30,31,30,31,31,30,31,30,31};
+
+    // boolean to test to see if we need to build date information or not
+    int dategiven = ((config.Value("startmonth")) ||
+		     (config.Value("startday"))   ||
+		     (config.Value("startyear"))  ||
+		     (config.Value("endmonth"))   ||
+		     (config.Value("endday"))     ||
+		     (config.Value("endyear")));
+
+    // find the end of time
+    endoftime = gmtime(&eternity);
+
+    if(dategiven)    // user specified some sort of date information
+      {
+	int reldate = ((config.Value("startmonth") < 0) ||
+		     (config.Value("startday") < 0)   ||
+		     (config.Value("startyear") < 0));
+	int t;
+
+	// set up the startdate structure
+	// see man mktime for details on the tm structure
+	startdate.tm_sec = 0;
+	startdate.tm_min = 0;
+	startdate.tm_hour = 0;
+	startdate.tm_yday = 0;
+	startdate.tm_wday = 0;
+
+	// The concept here is that if a user did not specify a part of a date,
+	// then we will make assumtions...
+	// For instance, suppose the user specified Feb, 1999 as the start
+	// range, we take steps to make sure that the search range date starts
+	// at Feb 1, 1999,
+	// along these same lines:  (these are in MM-DD-YYYY format)
+	// Startdates:      Date          Becomes
+	//                  01-01         01-01-1970
+	//                  01-1970       01-01-1970
+	//                  04-1970       04-01-1970
+	//                  1970          01-01-1970
+	// These things seem to work fine for start dates, as all months have
+	// the same first day however the ending date can't work this way.
+
+	if(config.Value("startday"))	// form input specified a start day
+	  {
+	    t = config.Value("startday");
+	    if (t < 0)
+	      {
+		time_t then = now + (t * (24*60*60));
+		lt = localtime(&then);
+		startdate.tm_mday = lt->tm_mday;
+		startdate.tm_mon = lt->tm_mon;
+		startdate.tm_year = lt->tm_year;
+	      }
+	    else
+		startdate.tm_mday = t;
+	    // tm days are 1 based, they are passed in as 1 based
+	  }
+	else if (!reldate)
+	    startdate.tm_mday = 1;	// otherwise, no start day, default to 1
+
+	if(config.Value("startmonth"))	// form input specified a start month
+	  {
+	    t = config.Value("startmonth");
+	    if (t < 0)
+		startdate.tm_mon += t;
+	    else
+		startdate.tm_mon = t - 1;
+	    // tm months are zero based.  They are passed in as 1 based
+	    while (startdate.tm_mon < 0)
+	      {
+		startdate.tm_mon += 12;
+		startdate.tm_year--;
+	      }
+	  }
+	else if (!reldate)
+	    startdate.tm_mon = 0;	// otherwise, no start month, default to 0
+
+	// year is handled a little differently... the tm_year structure
+	// wants the tm_year in a format of year - 1900.
+	// since we are going to convert these dates to a time_t,
+	// a time_t value of zero, the earliest possible date
+	// occurs Jan 1, 1970.  If we allow dates < 1970, then we
+	// could get negative time_t values right???
+	// (barring minor timezone offsets west of GMT, where Epoch is 12-31-69)
+
+	if(config.Value("startyear"))	// form input specified a start year
+	  {
+	    t = config.Value("startyear");
+	    if (t < 0)
+		startdate.tm_year += t;
+	    else
+	      {
+		startdate.tm_year = config.Value("startyear") - 1900;
+		if (startdate.tm_year < 69-1900) // correct for 2-digit years 00-68
+		    startdate.tm_year += 2000;	 //  - Gilles's fix
+		if (startdate.tm_year < 0)	 // correct for 2-digit years 69-99
+		    startdate.tm_year += 1900;
+	      }
+	  }
+	else if (!reldate)
+	    startdate.tm_year = 1970-1900;
+	     // otherwise, no start day, specify start at 1970
+
+	reldate = ((config.Value("endmonth") < 0) ||
+		     (config.Value("endday") < 0)   ||
+		     (config.Value("endyear") < 0));
+
+	// set up the enddate structure
+	enddate.tm_sec = 59;		// allow up to last second of end day
+	enddate.tm_min = 59;		//  - Gilles's fix
+	enddate.tm_hour = 23;
+	enddate.tm_yday = 0;
+	enddate.tm_wday = 0;
+
+	if(config.Value("endday") < 0)	// form input specified relative end day
+	  {
+	    // relative end day must be done before month or year
+	    t = config.Value("endday");
+	    time_t then = now + (t * (24*60*60));
+	    lt = localtime(&then);
+	    enddate.tm_mday = lt->tm_mday;
+	    enddate.tm_mon = lt->tm_mon;
+	    enddate.tm_year = lt->tm_year;
+	  }
+
+	if(config.Value("endmonth"))	// form input specified an end month
+	  {
+	    t = config.Value("endmonth");
+	    if (t < 0)
+		enddate.tm_mon += t;
+	    else
+		enddate.tm_mon = t - 1;
+	    // tm months are zero based.  They are passed in as 1 based
+	    while (enddate.tm_mon < 0)
+	      {
+		enddate.tm_mon += 12;
+		enddate.tm_year--;
+	      }
+	  }
+	else if (!reldate)
+	    enddate.tm_mon = 11;	// otherwise, no end month, default to 11
+
+	if(config.Value("endyear"))	// form input specified a end year
+	  {
+	    t = config.Value("endyear");
+	    if (t < 0)
+		enddate.tm_year += t;
+	    else
+	      {
+		enddate.tm_year = config.Value("endyear") - 1900;
+		if (enddate.tm_year < 69-1900)	// correct for 2-digit years 00-68
+		    enddate.tm_year += 2000;	//  - Gilles's fix
+		if (enddate.tm_year < 0)	// correct for 2-digit years 69-99
+		    enddate.tm_year += 1900;
+	      }
+	  }
+	else if (!reldate)
+	    enddate.tm_year = endoftime->tm_year;
+	     // otherwise, no end year, specify end at the end of time allowable
+
+	// Months have different number of days, and this makes things more
+	// complicated than the startdate range.
+	// Following the example above, here is what we want to happen:
+	// Enddates:        Date          Becomes
+	//                  04-31         04-31-endoftime->tm_year
+	//                  05-1999       05-31-1999, may has 31 days... we want to search until the end of may so...
+	//                  1999          12-31-1999, search until the end of the year
+
+	if(config.Value("endday") > 0)	// form input specified an end day
+	  {
+	    enddate.tm_mday = config.Value("endday");
+	    // tm days are 1 based, they are passed in as 1 based
+	  }
+	else if (!reldate)
+	  {
+	    // otherwise, no end day, default to the end of the month
+	    enddate.tm_mday = monthdays[enddate.tm_mon];
+	    if (enddate.tm_mon == 1)	// February, so check for leap year
+		if (((enddate.tm_year+1900) % 4 == 0 &&
+			    (enddate.tm_year+1900) % 100 != 0) ||
+		    (enddate.tm_year+1900) % 400 == 0)
+			enddate.tm_mday += 1;	// Feb. 29  - Gilles's fix
+	  }
+
+	// Convert the tm values into time_t values.
+	// Web servers specify modification times in GMT, but htsearch
+	// displays these modification times in the server's local time zone.
+	// For consistency, we would prefer to select based on this same
+	// local time zone.  - Gilles's fix
+
+	timet_startdate = mktime(&startdate);
+	timet_enddate = mktime(&enddate);
+
+	// I'm not quite sure what behavior I want to happen if
+	// someone reverses the start and end dates, and one of them is invalid.
+	// for now, if there is a completely invalid date on the start or end
+	// date, I will force the start date to time_t 0, and the end date to
+	// the maximum that can be handled by a time_t.
+
+	if(timet_startdate < 0)
+	    timet_startdate = 0;
+	if(timet_enddate < 0)
+	    timet_enddate = eternity;
+
+	// what if the user did something really goofy like choose an end date
+	// that's before the start date
+
+	if(timet_enddate < timet_startdate)  // if so, then swap them so they are in order
+	  {
+	    time_t timet_temp = timet_enddate;
+	    timet_enddate = timet_startdate;
+	    timet_startdate = timet_temp;
+	  }
+      }
+    else   // no date was specifed, so plug in some defaults
+      {
+	timet_startdate = 0;
+	timet_enddate = eternity;
+      }
+
+    // ... MG
+
     results->Start_Get();
     while ((id = results->Get_Next()))
     {
@@ -1022,6 +1372,7 @@ Display::buildMatchList()
 	// No special precations re: the option
 	// "uncoded_db_compatible" needs to be taken.
 	url = HtURLCodec::instance()->decode(coded_url);
+	HtURLRewriter::instance()->Replace(url);
 	if (!includeURL(url.get()))
 	{
 	    continue;
@@ -1029,7 +1380,7 @@ Display::buildMatchList()
 	
 
 	thisMatch = new ResultMatch();
-	thisMatch->setURL(url);
+	thisMatch->setURL(coded_url);
 	thisMatch->setRef(NULL);
 
 	//
@@ -1054,13 +1405,25 @@ Display::buildMatchList()
 	// We want older docs to have smaller values and the
 	// ultimate values to be a reasonable size (max about 100)
 
-	if (date_factor != 0.0 || backlink_factor != 0.0 || typ != SortByScore)
+	// New check added on whether or not we need to check date ranges - MG
+	if (date_factor != 0.0 || backlink_factor != 0.0 || typ != SortByScore
+	    || timet_startdate > 0 || enddate.tm_year < endoftime->tm_year)
 	  {
-	    DocumentRef *thisRef = docDB[thisMatch->getURL()];
+	    DocumentRef *thisRef = docDB.FindCoded(thisMatch->getURL());
 	    if (thisRef)   // We better hope it's not null!
 	      {
+		// code added by Mike Grommet for date search ranges
+		// check for valid date range. toss it out if it isn't relevant.
+		if(thisRef->DocTime() < timet_startdate ||
+		   thisRef->DocTime() > timet_enddate)
+		  {
+		    delete thisMatch;
+		    delete thisRef;
+		    continue;
+		  }
+
 		score += date_factor * 
-		  ((thisRef->DocTime() * 1000 / (double)time(0)) - 900);
+		  ((thisRef->DocTime() * 1000.0 / (double)now) - 900);
 		int links = thisRef->DocLinks();
 		if (links == 0)
 		  links = 1; // It's a hack, but it helps...
@@ -1129,7 +1492,8 @@ Display::excerpt(DocumentRef *ref, String urlanchor, int fanchor, int &first)
     // If previous conditions are false and "excerpt_show_top" is set to true
     // it shows the whole head. Else, it acts as default.
 
-    if (config.Boolean("excerpt_show_top", 0) || use_meta_description)
+    if (config.Boolean("excerpt_show_top", 0) || use_meta_description ||
+		!allWordsPattern->hasPattern())
       first = 0;
     else
       first = allWordsPattern->FindFirstWord(head, which, length);
@@ -1148,6 +1512,7 @@ Display::excerpt(DocumentRef *ref, String urlanchor, int fanchor, int &first)
 	}
     }
     else
+    if ( first == 0 || config.Value( "max_excerpts" ) == 1 )
     {
 	int	headLength = strlen(head);
 	int	length = config.Value("excerpt_length", 50);
@@ -1190,15 +1555,116 @@ Display::excerpt(DocumentRef *ref, String urlanchor, int fanchor, int &first)
 	    *text << config["end_ellipses"];
 	}
     }
+    else
+    {
+      *text = buildExcerpts( head, urlanchor, fanchor );
+    }
+
     return text;
 }
 
+//*****************************************************************************
+// Handle cases where multiple document excerpts are requested.
+//
+const String
+Display::buildExcerpts( char *head, String urlanchor, int fanchor )
+{
+  if ( !config.Boolean( "add_anchors_to_excerpt" ) )
+  {
+    fanchor = 0;
+  }
+
+  int    headLength    = strlen( head );
+  int    excerptNum    = config.Value( "max_excerpts", 1 );
+  int    excerptLength = config.Value( "excerpt_length", 50 );
+  int    lastPos       = 0;
+  int    curPos        = 0;
+
+  String text;
+
+  for ( int i = 0; i < excerptNum; ++i )
+  {
+    int which, termLength;
+
+    int nextPos = allWordsPattern->FindFirstWord( head + lastPos,
+                                                  which, termLength );
+
+    if ( nextPos < 0 )
+    {
+      // Ran out of matching terms
+      break;
+    }
+    else
+    {
+      // Determine offset from beginning of head
+      curPos = lastPos + nextPos;
+    }
+
+    // Slip a break in since there is another excerpt coming
+    if ( i != 0 )
+    {
+      text << "<br>\n";
+    }
+
+    // Determine where excerpt starts
+    char *start = &head[curPos] - excerptLength / 2;
+
+    if ( start < head )
+    {
+      start = head;
+    }
+    else
+    {
+      text << config["start_ellipses"];
+
+      while ( *start && HtIsStrictWordChar( *start ) )
+      {
+        start++;
+      }
+    }
+
+    // Determine where excerpt ends
+    char *end = start + excerptLength;
+
+    if ( end > head + headLength )
+    {
+      end = head + headLength;
+
+      text << hilight( start, urlanchor, fanchor );
+    }
+    else
+    {
+      while ( *end && HtIsStrictWordChar( *end ) )
+      {
+        end++;
+      }
+
+      // Save end char so that it can be restored
+      char endChar = *end;
+
+      *end = '\0';
+
+      text << hilight(start, urlanchor, fanchor);
+      text << config["end_ellipses"];
+
+      *end = endChar;
+    }
+
+    // No more words left to examine in head
+    if ( (lastPos = curPos + termLength) > headLength )
+     break;
+  }
+
+  return text;
+}
+
 //*****************************************************************************
 char *
 Display::hilight(char *str, String urlanchor, int fanchor)
 {
     static char		*start_highlight = config["start_highlight"];
     static char		*end_highlight = config["end_highlight"];
+    static char		*anchor_target = config["anchor_target"];
     static String	result;
     int			pos;
     int			which, length;
@@ -1206,13 +1672,19 @@ Display::hilight(char *str, String urlanchor, int fanchor)
     int			first = 1;
 
     result = 0;
-    while ((pos = allWordsPattern->FindFirstWord(str, which, length)) >= 0)
+    while (allWordsPattern->hasPattern() &&
+	   (pos = allWordsPattern->FindFirstWord(str, which, length)) >= 0)
     {
 	result.append(str, pos);
 	ww = (WeightWord *) (*searchWords)[which];
 	result << start_highlight;
 	if (first && fanchor)
-	    result << "<a href=\"" << urlanchor << "\">";
+	{
+	    result << "<a ";
+	    if (*anchor_target)
+		result << "target=\"" << anchor_target << "\" ";
+	    result << "href=\"" << urlanchor << "\">";
+	}
 	result.append(str + pos, length);
 	if (first && fanchor)
 	    result << "</a>";
