@@ -1,7 +1,7 @@
 //
 // Display.h
 //
-// $Id: Display.h,v 1.8.2.4 1999/11/24 05:17:10 grdetil Exp $
+// $Id: Display.h,v 1.8.2.6 2001/11/01 16:23:32 grdetil Exp $
 //
 //
 //
@@ -43,6 +43,7 @@ public:
 	
     void		display(int pageNumber);
     void		displayMatch(ResultMatch *, int current);
+    void		displayHTTPheaders();
     void		displayHeader();
     void		displayFooter();
     void		displayNomatch();
@@ -159,6 +160,7 @@ protected:
     void		expandVariables(char *);
     void		outputVariable(char *);
     String		*excerpt(DocumentRef *ref, String urlanchor, int fanchor, int &first);
+    const String        buildExcerpts( char *head, String urlanchor, int fanchor );
     char		*hilight(char *str, String urlanchor, int fanchor);
     void		setupTemplates();
     void		setupImages();
