@@ -8,7 +8,7 @@
 //
 //
 #if RELEASE
-static char RCSid[] = "$Id: htsearch.cc,v 1.24.2.9 2000/02/15 22:20:02 grdetil Exp $";
+static char RCSid[] = "$Id: htsearch.cc,v 1.24.2.18 2002/01/18 21:59:47 grdetil Exp $";
 #endif
 
 #include "htsearch.h"
@@ -25,6 +25,7 @@ static char RCSid[] = "$Id: htsearch.cc,v 1.24.2.9 2000/02/15 22:20:02 grdetil E
 #include <ctype.h>
 #include <signal.h>
 #include "HtURLCodec.h"
+#include "HtURLRewriter.h"
 #include "HtWordType.h"
 
 // If we have this, we probably want it.
@@ -46,6 +47,7 @@ void usage();
 
 int			debug = 0;
 int			minimum_word_length = 3;
+StringList		boolean_keywords;
 
 
 //*****************************************************************************
@@ -77,9 +79,18 @@ main(int ac, char **av)
  	switch (c)
  	{
  	    case 'c':
- 		configFile = optarg;
-                 override_config=1;
- 		break;
+	      // The default is obviously to do this securely
+	      // but if people want to shoot themselves in the foot...
+#ifndef ALLOW_INSECURE_CGI_CONFIG
+	      if (!getenv("REQUEST_METHOD"))
+		{
+#endif
+		  configFile = optarg;
+		  override_config=1;
+#ifndef ALLOW_INSECURE_CGI_CONFIG
+		}
+#endif
+	      break;
  	    case 'v':
  		debug++;
  		break;
@@ -102,24 +113,8 @@ main(int ac, char **av)
     //
     char	none[] = "";
     cgi		input(optind < ac ? av[optind] : none);
-
-    //
-    // Compile the URL limit pattern.
-    //
-    if (input.exists("restrict"))
-      {
-	char *sep = input["restrict"];
-	while ((sep = strchr(sep, '\001')) != NULL)
-	  *sep++ = '|';
-	limit_to.Pattern(input["restrict"]);
-    }
-    if (input.exists("exclude"))
-    {
-       char *sep = input["exclude"];
-       while ((sep = strchr(sep, '\001')) != NULL)
-       	 *sep++ = '|';
-       exclude_these.Pattern(input["exclude"]);
-    }
+    int		filenameok = (debug && getenv("REQUEST_METHOD") == 0);
+    String	filenamemsg;
 
     //
     // Setup the configuration database.  First we read the compiled defaults.
@@ -149,8 +144,9 @@ main(int ac, char **av)
     }
     if (access(configFile, R_OK) < 0)
     {
-	reportError(form("Unable to read configuration file '%s'",
-			 configFile.get()));
+	if (filenameok) filenamemsg << " '" << configFile.get() << "'";
+	reportError(form("Unable to read configuration file%s",
+			 filenamemsg.get()));
     }
     config.Read(configFile);
 
@@ -180,6 +176,27 @@ main(int ac, char **av)
     if (input.exists("sort"))
 	config.Add("sort", input["sort"]);
 
+    // Changes added 3-31-99, by Mike Grommet
+    // Check form entries for starting date, and ending date
+    // Each date consists of a month, day, and year
+
+    if (input.exists("startmonth"))
+	config.Add("startmonth", input["startmonth"]);
+    if (input.exists("startday"))
+	config.Add("startday", input["startday"]);
+    if (input.exists("startyear"))
+	config.Add("startyear", input["startyear"]);
+
+    if (input.exists("endmonth"))
+	config.Add("endmonth", input["endmonth"]);
+    if (input.exists("endday"))
+	config.Add("endday", input["endday"]);
+    if (input.exists("endyear"))
+	config.Add("endyear", input["endyear"]);
+
+    // END OF CHANGES BY MIKE GROMMET    
+
+
     minimum_word_length = config.Value("minimum_word_length", minimum_word_length);
 
     StringList form_vars(config["allow_in_form"], " \t\r\n");
@@ -189,6 +206,35 @@ main(int ac, char **av)
 	config.Add(form_vars[i], input[form_vars[i]]);
     }
  
+    //
+    // Compile the URL limit pattern.
+    //
+
+    StringList urllist;
+    String urlpat;
+
+    if (strlen(config["restrict"]))
+    {
+        // Create a temporary list from either the configuration
+        // file or the input parameter
+        urllist.Create(config["restrict"], "| \t\r\n\001");
+        urlpat = urllist.Join('|');
+        urllist.Release();  // release the temporary list of URLs
+        config.Add("restrict", urlpat);  // re-create the config attribute
+	limit_to.Pattern(urlpat);  // Set the new limit pattern
+    }
+
+    if (strlen(config["exclude"]))
+    {
+        // Create a temporary list from either the configuration
+        // file or the input parameter
+        urllist.Create(config["exclude"], "| \t\r\n\001");
+        urlpat = urllist.Join('|');
+        urllist.Release();  // release the temporary list of URLs
+        config.Add("exclude", urlpat);  // re-create the config attribute
+        exclude_these.Pattern(urlpat);
+    }
+
     // Ctype-like functions for what constitutes a word.
     HtWordType::Initialize(config);
 
@@ -201,6 +247,21 @@ main(int ac, char **av)
       reportError(form("Invalid url_part_aliases or common_url_parts: %s",
                        url_part_errors.get()));
 
+    // for htsearch, we use search_rewrite_rules attribute for HtURLRewriter.
+    config.AddParsed("url_rewrite_rules", "${search_rewrite_rules}");
+    url_part_errors = HtURLRewriter::instance()->ErrMsg();
+    if (url_part_errors.length() != 0)
+      reportError(form("Invalid url_rewrite_rules: %s",
+                       url_part_errors.get()));
+
+    // Load boolean_keywords from configuration
+    // they should be placed in this order:
+    //    0       1       2
+    //    and     or      not
+    boolean_keywords.Create(config["boolean_keywords"], "| \t\r\n\001");
+    if (boolean_keywords.Count() != 3)
+	reportError("boolean_keywords attribute is not correctly specified");
+
     Parser	*parser = new Parser();
 	
     //
@@ -248,29 +309,33 @@ main(int ac, char **av)
     String	word_db = config["word_db"];
     if (access(word_db, R_OK) < 0)
     {
-	reportError(form("Unable to read word database file '%s'\nDid you run htmerge?",
-			 word_db.get()));
+	if (filenameok) filenamemsg << " '" << word_db.get() << "'";
+	reportError(form("Unable to read word database file%s\nDid you run htmerge?",
+			 filenamemsg.get()));
     }
     ResultList	*results = htsearch(word_db, searchWords, parser);
 
     String	index = config["doc_index"];
     if (access(index, R_OK) < 0)
     {
-	reportError(form("Unable to read document index file '%s'\nDid you run htmerge?",
-			 index.get()));
+	if (filenameok) filenamemsg << " '" << index.get() << "'";
+	reportError(form("Unable to read document index file%s\nDid you run htmerge?",
+			 filenamemsg.get()));
     }
     String	doc_db = config["doc_db"];
     if (access(doc_db, R_OK) < 0)
     {
-	reportError(form("Unable to read document database file '%s'\nDid you run htmerge?",
-			 doc_db.get()));
+	if (filenameok) filenamemsg << " '" << doc_db.get() << "'";
+	reportError(form("Unable to read document database file%s\nDid you run htmerge?",
+			 filenamemsg.get()));
     }
 
     Display	display(index, doc_db);
     if (display.hasTemplateError())
       {
-	reportError(form("Unable to read template file '%s'\nDoes it exist?",
-                         config["template_name"]));
+	if (filenameok) filenamemsg << " '" << config["template_name"] << "'";
+	reportError(form("Unable to read template file%s\nDoes it exist?",
+			 filenamemsg.get()));
 	return 0;
       }
     display.setOriginalWords(originalWords);
@@ -305,11 +370,11 @@ createLogicalWords(List &searchWords, String &logicalWords, String &wm)
 	if (!ww->isHidden)
 	{
 	    if (strcmp(ww->word, "&") == 0 && wasHidden == 0)
-		logicalWords << " and ";
+		logicalWords << " " << boolean_keywords[0] << " ";
 	    else if (strcmp(ww->word, "|") == 0 && wasHidden == 0)
-		logicalWords << " or ";
+		logicalWords << " " << boolean_keywords[1] << " ";
 	    else if (strcmp(ww->word, "!") == 0 && wasHidden == 0)
-		logicalWords << " not ";
+		logicalWords << " " << boolean_keywords[2] << " ";
 	    else if (wasHidden == 0)
 	    {
 		logicalWords << ww->word;
@@ -379,6 +444,9 @@ setupWords(char *allWords, List &searchWords, int boolean, Parser *parser,
     String		word;
     // Why use a char type if String is the new char type!!!
     char		*prefix_suffix = config["prefix_match_character"];
+    char		*wildcard = prefix_suffix;
+    if (*wildcard == '\0')
+	wildcard = "*";
     while (*pos)
     {
 	while (1)
@@ -396,12 +464,12 @@ setupWords(char *allWords, List &searchWords, int boolean, Parser *parser,
 		tempWords.Add(new WeightWord(s, -1.0));
 		break;
 	    }
-	    else if (HtIsWordChar(t) || t == ':' ||
+	    else if (HtIsWordChar(t) || t == ':' || t == *wildcard ||
 			 (strchr(prefix_suffix, t) != NULL) || (t >= 161 && t <= 255))
 	    {
 		word = 0;
-		while (t && (HtIsWordChar(t) ||
-			     t == ':' || (strchr(prefix_suffix, t) != NULL) || (t >= 161 && t <= 255)))
+		while (t && (HtIsWordChar(t) || t == ':' || t == *wildcard ||
+			     (strchr(prefix_suffix, t) != NULL) || (t >= 161 && t <= 255)))
 		{
 		    word << (char) t;
 		    t = *pos++;
@@ -409,18 +477,22 @@ setupWords(char *allWords, List &searchWords, int boolean, Parser *parser,
 
 		pos--;
 	  	word.lowercase();
-		if (boolean && mystrcasecmp(word.get(), "and") == 0)
+		if (boolean && mystrcasecmp(word.get(), boolean_keywords[0]) == 0)
 		{
 		    tempWords.Add(new WeightWord("&", -1.0));
 		}
-		else if (boolean && mystrcasecmp(word.get(), "or") == 0)
+		else if (boolean && mystrcasecmp(word.get(), boolean_keywords[1]) == 0)
 		{
 		    tempWords.Add(new WeightWord("|", -1.0));
 		}
-		else if (boolean && mystrcasecmp(word.get(), "not") == 0)
+		else if (boolean && mystrcasecmp(word.get(), boolean_keywords[2]) == 0)
 		{
 		    tempWords.Add(new WeightWord("!", -1.0));
 		}
+		else if (strcmp(word.get(), wildcard) == 0)
+		{
+		    tempWords.Add(new WeightWord(wildcard, 1.0));
+		}
 		else
 		{
 		    // Add word to excerpt matching list
@@ -475,7 +547,7 @@ setupWords(char *allWords, List &searchWords, int boolean, Parser *parser,
     // configuration attribute.
     // For algorithms other than exact, we need to also do word lookups.
     //
-    StringList	algs(config["search_algorithm"], " \t,");
+    StringList	algs(config["search_algorithm"], " \t");
     List		algorithms;
     String		name, weight;
     double		fweight;
@@ -660,13 +732,32 @@ htsearch(char *wordfile, List &searchWords, Parser *parser)
 void
 addRequiredWords(List &searchWords, StringList &requiredWords)
 {
-    searchWords.Insert(new WeightWord("(", -1.0), 0);
-    searchWords.Add(new WeightWord(")", -1.0));
-
-    for (int i = 0; i < requiredWords.Count(); i++)
+    static int	any_keywords = config.Boolean("any_keywords", 0);
+    if (requiredWords.Count() == 0)
+	return;
+    if (searchWords.Count() > 0)
     {
+	searchWords.Insert(new WeightWord("(", -1.0), 0);
+	searchWords.Add(new WeightWord(")", -1.0));
 	searchWords.Add(new WeightWord("&", -1.0));
-	searchWords.Add(new WeightWord(requiredWords[i], 1.0));
+    }
+    if (requiredWords.Count() == 1)
+    {
+	searchWords.Add(new WeightWord(requiredWords[0], 1.0));
+    }
+    else
+    {
+	searchWords.Add(new WeightWord("(", -1.0));
+	searchWords.Add(new WeightWord(requiredWords[0], 1.0));
+	for (int i = 1; i < requiredWords.Count(); i++)
+	{
+	    if (any_keywords)
+		searchWords.Add(new WeightWord("|", -1.0));
+	    else
+		searchWords.Add(new WeightWord("&", -1.0));
+	    searchWords.Add(new WeightWord(requiredWords[i], 1.0));
+	}
+	searchWords.Add(new WeightWord(")", -1.0));
     }
 }
 
