@@ -5,14 +5,18 @@
 //
 //
 #if RELEASE
-static char RCSid[] = "$Id: parser.cc,v 1.6.2.3 1999/09/01 20:16:36 grdetil Exp $";
+static char RCSid[] = "$Id: parser.cc,v 1.6.2.6 2002/01/18 21:59:47 grdetil Exp $";
 #endif
 
 #include "parser.h"
+#include "QuotedStringList.h"
 
 #define	WORD	1000
 #define	DONE	1001
 
+extern StringList	boolean_keywords;
+QuotedStringList	boolean_syntax_errors;
+
 
 //*****************************************************************************
 Parser::Parser()
@@ -33,6 +37,14 @@ Parser::Parser()
 int
 Parser::checkSyntax(List *tokenList)
 {
+    void	reportError(char *);
+    // Load boolean_syntax_errors from configuration
+    // they should be placed in this order:
+    // 0        1               2            3            4
+    // Expected "a search word" "at the end" "instead of" "end of expression"
+    boolean_syntax_errors.Create(config["boolean_syntax_errors"], "| \t\r\n\001");
+    if (boolean_syntax_errors.Count() != 5)
+	reportError("boolean_syntax_errors attribute is not correctly specified");
     tokens = tokenList;
     valid = 1;
     fullexpr(0);
@@ -48,7 +60,7 @@ Parser::fullexpr(int output)
     expr(output);
     if (valid && lookahead != DONE)
     {
-	setError("end of expression");
+	setError(boolean_syntax_errors[4]);
     }
 }
 
@@ -100,7 +112,10 @@ Parser::expr(int output)
     }
     if (valid && lookahead == WORD)
     {
-	setError("'AND' or 'OR'");
+	String	expected = "'";
+	expected << boolean_keywords[0] << "' " << boolean_keywords[1] << " '"
+		 << boolean_keywords[1] << "'";
+	setError(expected.get());
     }
 }
 
@@ -152,7 +167,7 @@ Parser::factor(int output)
     }
     else
     {
-	setError("a search word");
+	setError(boolean_syntax_errors[1]);
     }
 }
 
@@ -177,20 +192,26 @@ Parser::setError(char *expected)
     {
 	valid = 0;
 	error = 0;
-	error << "Expected " << expected;
+	error << boolean_syntax_errors[0] << ' ' << expected;
 	if (lookahead == DONE || !current)
 	{
-	    error << " at the end";
+	    error << ' ' << boolean_syntax_errors[2];
 	}
 	else
 	{
-	    error << " instead of '" << current->word.get();
-	    error << '\'';
+	    error << ' ' << boolean_syntax_errors[3] << " '"
+		  << current->word.get() << "'";
 	    switch (lookahead)
 	    {
-	    case '&':	error << " or 'AND'";	break;
-	    case '|':	error << " or 'OR'";	break;
-	    case '!':	error << " or 'NOT'";	break;
+	    case '&':	error << ' ' << boolean_keywords[1] << " '"
+			      << boolean_keywords[0] << "'";
+			break;
+	    case '|':	error << ' ' << boolean_keywords[1] << " '"
+			      << boolean_keywords[1] << "'";
+			break;
+	    case '!':	error << ' ' << boolean_keywords[1] << " '"
+			      << boolean_keywords[2] << "'";
+			break;
 	    }
 	}
     }
@@ -219,6 +240,24 @@ Parser::perform_push()
 	list->isIgnore = 1;
 	return;
     }
+    static char	*wildcard = config["prefix_match_character"];
+    if (*wildcard == '\0')
+	wildcard = "*";
+    if (strcmp(temp.get(), wildcard) == 0) {
+	Database	*docIndex = Database::getDatabaseInstance();
+	docIndex->OpenRead(config["doc_index"]);
+	docIndex->Start_Get();
+	while ((p = docIndex->Get_Next()))
+	{
+	    dm = new DocMatch;
+	    dm->score = current->weight;
+	    dm->id = atoi(p);
+	    dm->anchor = 0;
+	    list->add(dm);
+	}
+	delete docIndex;
+	return;
+    }
     temp.lowercase();
     p = temp.get();
     if (temp.length() > maximum_word_length)
@@ -325,6 +364,7 @@ Parser::perform_and(int isand)
 void
 Parser::perform_or()
 {
+    static int	multimatch_factor = config.Value("multimatch_factor", 1);
     ResultList		*l1 = (ResultList *) stack.pop();
     ResultList		*result = (ResultList *) stack.peek();
     int			i;
@@ -378,6 +418,8 @@ Parser::perform_or()
 	    // Duplicate document.  We just need to add the scores together
 	    //
 	    dm2->score += dm->score;
+	    // Boost matches that contain more than one word...
+	    dm2->score *= multimatch_factor;
 	    if (dm->anchor < dm2->anchor)
 		dm2->anchor = dm->anchor;
 	}
@@ -407,9 +449,11 @@ Parser::parse(List *tokenList, ResultList &resultMatches)
     ResultList	*result = (ResultList *) stack.pop();
     if (!result)  // Ouch!
       {
+	if (!valid)
+	    return;
 	valid = 0;
 	error = 0;
-	error << "Expected to have something to parse!";
+	error << boolean_syntax_errors[0] << ' ' << boolean_syntax_errors[1];
 	return;
       }
     List		*elements = result->elements();
